package org.pathwaycommons.trans;

import java.io.*;
import java.util.*;

import org.jdom.Namespace;
import org.jdom.Element;
import org.jdom.JDOMException;

import org.genemania.dw.entity.ExtResource;
import org.genemania.dw.tools.IdentifierMapperService;
import org.genemania.dw.util.DWUtil;
import org.genemania.dw.util.DefParams;
import org.genemania.dw.util.GenUtil;
import org.pathwaycommons.trans.util.XMLElementInfo;
import org.pathwaycommons.trans.util.XMLUtil;
import org.pathwaycommons.trans.util.GI_information;
import org.pathwaycommons.trans.util.LoadGIInformation;

/**
 * BIND to PSI-MI 2.5 xml translator. 
 * Source file names are presumed to be in the format: 'taxid9606.1.*'
 * mapXref () currently serves GI & EntrezID only. Assumes the source ID is
 * the primary ref, and the only new ref supported is Uniprot. Assumes GI is
 * the primary ref for BIND.
 *
 * @author rashadbadrawi
 */

public class BINDToPSIMITrans extends PSIMITrans {

    //local atts, reflecting build preferences
    private boolean proteinAFlag;
    private boolean proteinBFlag;
    private int cntProteinA;
    private int cntProteinB;

    private HashMap <String, Integer> interactionTypeCount = new HashMap<String, Integer> ();
    private boolean transComplexesFlag;
    private boolean mapIDsFlag;
    private boolean transProOnlyFlag;
    private boolean cleanDataFlag;

    private int cntTransInteraction;
    private int cntNonTransInteraction;
    private int cntTotalInteraction;
    private int cntBadInteraction;
    private String BINDInteractionID;                   //easier handling
    //atts used down the hierarchy
    protected TreeMap <String, String> interactionsMap = new TreeMap <String, String> ();
    protected TreeMap <String, String> badInteractionsMap = new TreeMap <String, String> ();
    protected ArrayList <String> speciesList = new ArrayList <String> ();
    protected String fileName;
    protected BufferedWriter bw;
    protected  String inputFilesPath;
    protected Namespace psins;

    //create some other stats storage files
    protected BufferedWriter Novel_GO_terms;
    protected BufferedWriter missing_pmids;
    protected BufferedWriter error_log;
    protected BufferedWriter stats_log;
    protected BufferedWriter interaction_conversions_log;

    protected MissingPMIDsFile new_pmids;

    //everything needs to have a unique identifier so change this to one counter
    //for each object
    //TODO: don't need separte counter but instead need the one counter that each of the PSI objects use and increment.
    private int counter = 1;


    //complex path
    String COMPLEX_PATH_PROP = "./Tools/Interactions_for_Complexes/";

    //Index of all the interactions in the current build
    //protected HashMap<String, Element> interaction_index ;

    //index of all bind interaction ids.
    //protected HashMap<String, String> bindInteractionId_index;

    //index of the unique gis in all of BIND
    protected HashMap<String, Integer> GI_index;

    //index of information for each GI
    protected HashMap<Integer, GI_information> GI_history;

    //iondex of the interaction id s that need to be saved.
    protected HashSet<String> interactionid_incomplexes;

    protected int currentTaxID = -1;
    protected boolean badInteractionData = false;

    protected BINDToPSIMITrans () {}

    protected BINDToPSIMITrans (String inputFilesPath) throws Exception {

        log = GenUtil.getDefaultLog();
        setInputPath (inputFilesPath);
        setMapIDsFlag (new Boolean (
          DefParams.getDefaultProp(DefParams.BIND_MAP_ID_PROP)).booleanValue());
        setTransComplexes (new Boolean (
          DefParams.getDefaultProp(DefParams.BIND_TRANS_COM_PROP)).booleanValue());
        setTransProOnly (new Boolean (
          DefParams.getDefaultProp(DefParams.BIND_TRANS_PRO_PROP)).booleanValue());
        setCleanData (new Boolean (
          DefParams.getDefaultProp(DefParams.BIND_CLEAN_DATA_PROP)).booleanValue());
        //global setters
        XMLElementInfo.readFromFile();
        CVMapEntry.loadCVMappings(
                  DefParams.getDefaultProp (DefParams.BIND_PSIMI_CV_FILE_PROP),
                  PSIMITags.BIND_SL, PSIMITags.DB_PSIMI);

        //load missing pubmed information file
        new_pmids = new MissingPMIDsFile("./resources/title2PMID_output.txt");
        new_pmids.loadMissingPMIDsFile();

        //create file to store the references to the missing pmids
        File missingPmidsFile = new File(DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP), MISSING_PMIDS);
        missing_pmids = new BufferedWriter(new FileWriter(missingPmidsFile));

        //create file to store the new GO terms found in the file.
        File novelGoTermsFile = new File(DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP), NOVEL_GO_TERMS);
        Novel_GO_terms = new BufferedWriter(new FileWriter(novelGoTermsFile));

        File errors = new File(DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP), ERROR_LOG);
        error_log = new BufferedWriter(new FileWriter(errors));

        //create a file to store all the stats from the buils
        File basic_stats = new File(DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP), "basic_Stats.txt");
        stats_log = new BufferedWriter(new FileWriter(basic_stats));

        File conversions = new File(DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP), "interaction_conversion_log.txt");
        interaction_conversions_log = new BufferedWriter(new FileWriter(conversions));

        //interaction_index = new HashMap<String, Element>();

        //bindInteractionId_index = new HashMap<String,String>();

        GI_index = new HashMap<String, Integer>();

        LoadGIInformation gi_reader = new LoadGIInformation("./resources/unique_gis_history.txt");
        GI_history = gi_reader.load();

        interactionid_incomplexes = new HashSet<String>();
        //put in a dummy just in case there are no complexes
        interactionid_incomplexes.add("1");

    }

    private void setInputPath (String inputFilesPath) {

        this.inputFilesPath = inputFilesPath;
    }

    private void setMapIDsFlag (boolean mapIDsFlag) {

        this.mapIDsFlag = mapIDsFlag;
    }

    private void setTransComplexes (boolean transComplexes) {

        this.transComplexesFlag = transComplexes;
    }

    private void setTransProOnly (boolean transProOnly) {

        this.transProOnlyFlag = transProOnly;
    }

    private void setCleanData (boolean cleanDataFlag) {

        this.cleanDataFlag = cleanDataFlag;
    }

    public String getInputFilesPath () { return inputFilesPath; }

    public boolean isTransComplexes () { return this.transComplexesFlag; }

    public boolean isMapIDs () { return this.mapIDsFlag; }

    public boolean isTransProsOnly () { return this.transProOnlyFlag; }

    public boolean isCleanData () { return this.cleanDataFlag; }

    public void translate () throws Exception  {

        System.out.println ("Translation type: " + PSIMITrans.BIND_TO_PSIMI25);
        System.out.println ("Translating input: " + getInputFilesPath ());
        File sourceFileOrDir = new File (getInputFilesPath());
        if (!isMapIDs()) {
            System.out.println ("Rashad ID mapping service off. ID1Fetch mapping ON");

            //load id_mapping file.
        }
        File badDataFile = new File (
             DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP), BAD_ENTRIES);
        bw = new BufferedWriter (new FileWriter (badDataFile));

        String outputFileName;
        if (sourceFileOrDir.isFile ()) {
            fileName = sourceFileOrDir.getAbsolutePath();
            this.currentTaxID = Integer.parseInt (fileName.substring(
                                fileName.indexOf(BINDTags.SOURCE_FILE_PREFIX) +
                                BINDTags.SOURCE_FILE_PREFIX.length (),
                                fileName.indexOf(GenUtil.DOT)));
            addSpecies (this.currentTaxID);
            if (isMapIDs()) {
                IdentifierMapperService.flush(
                                new String [] {ExtResource.LIST_UNIPROT},
                                new int [] {this.currentTaxID});
                IdentifierMapperService.init(
                        new String [] {ExtResource.LIST_UNIPROT},
                        new int [] {this.currentTaxID});
            }
            Element rootElement = XMLUtil.readSchemaBasedXML(fileName);
            Element PSIRootElement = translateBIND (null, rootElement);
            outputFileName = sourceFileOrDir.getName();
            //if (outputFileName.endsWith(XMLUtil.XML_FILE_SUFFIX)) {
            if (outputFileName.contains (XMLUtil.XML_FILE_SUFFIX)) {
                //hopefully, only one
                outputFileName = outputFileName.replace (XMLUtil.XML_FILE_SUFFIX,
                        PSIMI_FILE_SUFFIX + XMLUtil.XML_FILE_SUFFIX);
            }
            outputFileName = new File ( new File (DefParams.getDefaultProp(
                             DefParams.TOOLS_PATH_PROP)).getAbsolutePath(),
                             outputFileName).getAbsolutePath();
            XMLUtil.dump (PSIRootElement, outputFileName);
            rootElement = null;                         //attempt cleaning up
            PSIRootElement = null;
            System.gc();
        } else {            //} else if (sourceFileOrDir.isDirectory () {
            File filesArr [] = sourceFileOrDir.listFiles();
            Arrays.sort (filesArr);
            Element PSIRootElement = null;
            int cnt = 1;
            for (int i = 0; i < filesArr.length; i++) {

                if (!filesArr [i].getAbsolutePath().contains(XMLUtil.XML_FILE_SUFFIX)) {
                    continue;
                }
                fileName = filesArr [i].getAbsolutePath();
                System.out.println ("Processing file: " + cnt++ + " " + fileName);
                Element rootElement = XMLUtil.readSchemaBasedXML(fileName);
                int newTaxID = Integer.parseInt (fileName.substring(
                                    fileName.indexOf(BINDTags.SOURCE_FILE_PREFIX) +
                                    BINDTags.SOURCE_FILE_PREFIX.length (),
                                    fileName.indexOf(GenUtil.DOT)));
                //if this is a new taxonomy.
                if (newTaxID != this.currentTaxID) {
                    if(isMapIDs()) {
                        IdentifierMapperService.flush(
                                  new String [] {ExtResource.LIST_UNIPROT},
                                  new int [] {this.currentTaxID});
                    }
                    if (PSIRootElement != null) {
                        outputFileName = new File (
                                   new File (DefParams.getDefaultProp(
                                   DefParams.TOOLS_PATH_PROP)).getAbsolutePath(),
                                   BINDTags.SOURCE_FILE_PREFIX + this.currentTaxID +
                                   PSIMI_FILE_SUFFIX + XMLUtil.XML_FILE_SUFFIX).getAbsolutePath();
                        XMLUtil.dump (PSIRootElement, outputFileName);
                        PSIRootElement = null;
                        outputFileName = null;
                        //temp - reducing overhead, but prohibits 'some'
                        //inter-species interaction referencing
                        //interaction_index.clear();
                        interactionsMap.clear();
                        badInteractionsMap.clear();
                    }
                    addSpecies (this.currentTaxID);
                    this.currentTaxID = newTaxID;
                    System.gc();
                }

                if (isMapIDs()) {
                    IdentifierMapperService.init(
                              new String [] {ExtResource.LIST_UNIPROT},
                              new int [] {this.currentTaxID});
                }
                PSIRootElement = translateBIND (PSIRootElement, rootElement);
                rootElement = null;                      //attempt cleaning up
                System.gc();
           }
           //to handle the last set
           if (PSIRootElement != null) {
               outputFileName = new File (
                                   new File (DefParams.getDefaultProp(
                                   DefParams.TOOLS_PATH_PROP)).getAbsolutePath(),
                                   BINDTags.SOURCE_FILE_PREFIX + this.currentTaxID +
                                   PSIMI_FILE_SUFFIX + XMLUtil.XML_FILE_SUFFIX).getAbsolutePath();
               XMLUtil.dump (PSIRootElement, outputFileName);
               PSIRootElement = null;
               System.gc();
           }
        }
        /*
        if (isMapIDs()) {
            IdentifierMapperService.dumpUnmatchedUni(ExtResource.LIST_GI);
            IdentifierMapperService.dumpUnmatchedUni(ExtResource.LIST_ID);
        }
        */

        //output the total number of unique BINDIds
        //stats_log.write ("\nTotal unique bind identifiers: " + bindInteractionId_index.size() + "\n" );

        //output all the unique GIs to a file
        for(String s:GI_index.keySet())
            Novel_GO_terms.write(s + "\t" + GI_index.get(s) + "\n");

        //close all the files
        this.Novel_GO_terms.flush();
        this.Novel_GO_terms.close();

        this.missing_pmids.flush();
        this.missing_pmids.close();

        this.error_log.flush();
        this.error_log.close();

        this.bw.flush();
        this.bw.close ();

        this.stats_log.flush() ;
        this.stats_log.close();
        this.interaction_conversions_log.flush();
        this.interaction_conversions_log.close();

        //output the complete set of interactions
        //create file to store the new GO terms found in the file.
        File InteractionStatsFile = new File(DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP), "interaction_stats");
        BufferedWriter interaction_Stats = new BufferedWriter(new FileWriter(InteractionStatsFile));
        interaction_Stats.write(interactionTypeCount.toString());
        interaction_Stats.flush();
        interaction_Stats.close();

        listSpecies ();
    }

    //This method : indexes all the interactions in all the files that it goes through
    // and extracts interactions that are needed for complex construction and outputs them to a newly formed
    // xml file for separate processing.
    public void extractAllComplexInteractions() throws Exception{

        //the index of all interactions.
        HashMap<String, String> index = new HashMap<String, String>();
        //the set of interactions for each species that needs to be saved.
        HashMap<Integer,HashSet<String>> species_interactions = new HashMap<Integer,HashSet<String>>();

        //iondex of the interaction id s that need to be saved for an indivdual species
        HashSet<String> interactions_incomplexes = new HashSet<String>();

        BINDToPSIMITransComplex BINDComTrans = new BINDToPSIMITransComplex ();

        System.out.println ("Translating input: " + getInputFilesPath ());
        File sourceFileOrDir = new File (getInputFilesPath());

        //if the input is a single file - process it
        if (sourceFileOrDir.isFile ()) {
            fileName = sourceFileOrDir.getAbsolutePath();
            this.currentTaxID = Integer.parseInt (fileName.substring(
                                   fileName.indexOf(BINDTags.SOURCE_FILE_PREFIX) +
                                   BINDTags.SOURCE_FILE_PREFIX.length (),
                                   fileName.indexOf(GenUtil.DOT)));
            addSpecies (this.currentTaxID);

            //load the XML into memory
            Element rootElement = XMLUtil.readSchemaBasedXML(fileName);

            //scan complexes and add all interaction ids that are found in complexes
            BINDComTrans.scanComplexes(interactions_incomplexes, rootElement);

            //create an index of where the files where all the interactions are found
            indexInteractions(rootElement,sourceFileOrDir.getAbsolutePath(), index);
            rootElement = null;
            System.gc();

       //if the input is a directory - process all the files.
        } else {
            File filesArr [] = sourceFileOrDir.listFiles();
            Arrays.sort (filesArr);
            int cnt = 1;
            for (int i = 0; i < filesArr.length; i++) {
                if (!filesArr [i].getAbsolutePath().contains(XMLUtil.XML_FILE_SUFFIX)) {
                       continue;
                }
                fileName = filesArr [i].getAbsolutePath();
                System.out.println ("Processing file: " + cnt++ + " " + fileName);
                Element rootElement = XMLUtil.readSchemaBasedXML(fileName);
                int newTaxID = Integer.parseInt (fileName.substring(
                                       fileName.indexOf(BINDTags.SOURCE_FILE_PREFIX) +
                                       BINDTags.SOURCE_FILE_PREFIX.length (),
                                       fileName.indexOf(GenUtil.DOT)));

                //if this taxid is different from the previous file
                if ((newTaxID != this.currentTaxID) && (this.currentTaxID != -1)) {

                    //add the previous taxid information to the set of interactions needed for
                    HashSet<String> interactions = new HashSet<String>();
                    interactions.addAll(interactions_incomplexes);
                    species_interactions.put(this.currentTaxID,interactions);

                    addSpecies (this.currentTaxID);
                    this.currentTaxID = newTaxID;
                    interactions_incomplexes.clear();
                    System.gc();
                }

                //scan complexes and add all interaction ids that are found in complexes
                BINDComTrans.scanComplexes(interactions_incomplexes, rootElement);

                //create an index of where the files where all the interactions are found
                indexInteractions(rootElement,filesArr [i].getAbsolutePath(),index);
                rootElement = null;
                System.gc();

                if(this.currentTaxID == -1)
                    this.currentTaxID = newTaxID;
              }//for loop going through each file - indexing all the interactions

           }

            //add the last set.
            //add the previous taxid information to the set of interactions needed for
            HashSet<String> interactions = new HashSet<String>();
            interactions.addAll(interactions_incomplexes);
            species_interactions.put(this.currentTaxID,interactions);


            //output the index to a file to be loaded if needed.
            //create a file to store the interactions needed for all the complexes of this species
            BufferedWriter bw_index = new BufferedWriter (new FileWriter ( new File (COMPLEX_PATH_PROP, "interaction_index.txt")));
            for(Iterator iterator_index = index.keySet().iterator(); iterator_index.hasNext();){
                String current_key = (String)iterator_index.next();
                bw_index.write(current_key + "\t" + index.get(current_key) + "\n");
            }
            bw_index.close();

            //we have
            // index - an index of where every interaction is found
           //  species_interactions - the set of interactions in complexes for each species.

            //Go through each species set and create a file with the interactions that will be needed for complex processing.
            Iterator iterator_species = species_interactions.keySet().iterator();
            while(iterator_species.hasNext()) {

                //current species
                Integer speciesID = (Integer)iterator_species.next ();

                //get the set of interactions associated with this species
                HashSet<String> interaction_subset = species_interactions.get(speciesID);

                //only create a file if there are interactions in the set of interactions needed for the complexes.
                if(!interaction_subset.isEmpty()){
                    //create a file to store the interactions needed for all the complexes of this species
                    BufferedWriter bw = new BufferedWriter (new OutputStreamWriter( new FileOutputStream( new File (COMPLEX_PATH_PROP, "taxid" + speciesID + ".interactions_for_complexes.xml")),"UTF8"));

                    //write a header to the file containing standard element for Bind
                    bw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                        "<BIND-Submit>\n" +  "<BIND-Submit_date>\n" +  "<Date>\n" +
                        "<Date_str>Fri Jul 21 10:01:47 EDT 2006</Date_str>\n" + "</Date>\n" +
                        "</BIND-Submit_date>\n" + "<BIND-Submit_sub>\n" +
                        "<BIND-Submitter>\n" + "<BIND-Submitter_contact>\n" +
                        "<BIND-Contact-info/>\n" + "</BIND-Submitter_contact>\n" +
                        "<BIND-Submitter_subtype value=\"new\"/>\n" +"</BIND-Submitter>\n" +
                        "</BIND-Submit_sub>\n" +"<BIND-Submit_interactions>\n" +
                        "<BIND-Interaction-set>\n" + "<BIND-Interaction-set_interactions>");

                    //get the files we need to load.(with each file associate the number of interactions in that file)
                    //neeeded files, key = file needed, object is a set of interactions from that file that are needed.
                    HashMap<String, HashSet<String>> needed_files = new HashMap<String, HashSet<String>>();

                    Iterator iterator_interactions = interaction_subset.iterator();
                    while(iterator_interactions.hasNext()){

                        //current interaction
                        String current_interaction = (String)iterator_interactions.next();
                        //get the filename where this interaction is found
                        String current_filename = index.get(current_interaction);

                        HashSet<String> subset;
                        //if we are alreaday tracking this file get the its set of interaction
                        if(needed_files.containsKey(current_filename))
                            subset = needed_files.get(current_filename);
                        //otherwise we haven't seen this file before, create an empty set of interactions
                        else
                            subset = new HashSet<String>();

                        subset.add(current_interaction);
                        needed_files.put(current_filename, subset);

                    }

                    //System.out.println("needed files"+needed_files.toString());

                    //for each needed files get the interactions we need
                    //put the needed interactions in a separate file

                    Iterator iterator_files = needed_files.keySet().iterator();
                    while(iterator_files.hasNext()){
                        String file = (String)iterator_files.next();
                        getInteractions(file, needed_files.get(file),bw);
                    }

                    //write the footer standard for the BIND file
                    bw.write("</BIND-Interaction-set_interactions>\n" + "</BIND-Interaction-set>\n" +
                        "</BIND-Submit_interactions></BIND-Submit>");

                    bw.close();

                }
            }


   }

    private void indexInteractions(Element BINDSubmitE, String Filename, HashMap<String, String> index) throws Exception{

        ArrayList <Element> bindIntList = XMLUtil.getElement (BINDTags.INTERACTION,
                                           BINDSubmitE, 1);
        for (int i = 0; i < bindIntList.size (); i++) {


            //for each interaction create an index of the BIND ID to the interaction reference so
            //we can refer to it when converting the complexes.
             ArrayList <Element> eList = XMLUtil.getElement (BINDTags.INTERACTION_ID, bindIntList.get (i),
                                    XMLUtil.getDepth (BINDTags.INTERACTION));
            BINDInteractionID = null;
            if (XMLUtil.validateSingle (BINDTags.INTERACTION_ID, eList) &&
                XMLUtil.validateInt(eList.get (0))) {
                BINDInteractionID = eList.get (0).getTextTrim ();
            }

            //add id to the unique set of  BIND interaction ids
            index.put(BINDInteractionID, Filename);
        }
    }

    private void getInteractions(String Filename, HashSet<String> needed_interactions, BufferedWriter bw) throws Exception{

        System.out.println(Filename);

        //read file
        if(Filename == null){
            System.out.println("Filename is Null.  Can not find specified interactions.");
            return;
        }
        Element rootElement = XMLUtil.readSchemaBasedXML(Filename);

        ArrayList <Element> bindIntList = XMLUtil.getElement (BINDTags.INTERACTION,
                                           rootElement, 1);
        for (int i = 0; i < bindIntList.size (); i++) {

            //for each interaction create an index of the BIND ID to the interaction reference so
            //we can refer to it when converting the complexes.
             ArrayList <Element> eList = XMLUtil.getElement (BINDTags.INTERACTION_ID, bindIntList.get (i),
                                    XMLUtil.getDepth (BINDTags.INTERACTION));
            BINDInteractionID = null;
            if (XMLUtil.validateSingle (BINDTags.INTERACTION_ID, eList) &&
                XMLUtil.validateInt(eList.get (0))) {
                BINDInteractionID = eList.get (0).getTextTrim ();
            }

            //insert intearction ID and pointer into the hash map
            // //only put in HASH if it is in the list of IDs in complexes
            if(needed_interactions.contains(BINDInteractionID)){
                XMLUtil.dump(bindIntList.get (i),bw);
            }

        }

    }

    private Element translateBIND (Element entrySetE, Element BINDSubmitE)
                                   throws Exception {

        if (entrySetE == null) {
            entrySetE = setEntrySet ();
        }
        translateAllInteractions (entrySetE, BINDSubmitE);

        return entrySetE;
    }

    private Element setEntrySet () {

        this.psins = Namespace.getNamespace(PSIMITags.PSIMI_NS);
        Element entrySetE = new Element (PSIMITags.ENTRY_SET, this.psins);
        entrySetE.setAttribute(XMLUtil.SCHEMA_LOC_ATT, PSIMITags.SCHEMA_LOC,
                         Namespace.getNamespace(XMLUtil.XSI_NAME, XMLUtil.XSI));

        entrySetE.setAttribute(PSIMITags.LEVEL_ATT, PSIMITags.LEVEL);
        entrySetE.setAttribute (PSIMITags.VERSION_ATT, PSIMITags.VERSION);
        entrySetE.setAttribute (PSIMITags.MINOR_VERSION_ATT, PSIMITags.MINOR_VERSION);

        return entrySetE;
    }

    private Element setEntry () throws JDOMException {

        Element entryE = new Element (PSIMITags.ENTRY, this.psins);

        Element sourceE = new Element (PSIMITags.SOURCE, this.psins);
        setCVName (sourceE, PSIMITags.BIND_SL);

        Element bibRefE = new Element (PSIMITags.BIB, this.psins);
        CVMapEntry CVMapper = CVMapEntry.getMappedEntry (PSIMITags.PUBMED_SL);
        setXRef (bibRefE, PSIMITags.PREF, CVMapper.getPSIMITerm_SL(),
                 CVMapper.getPSIMITermID (), BINDTags.PUB_ID,
                 PSIMITags.getRefTypeAccn(PSIMITags.REFTYPE_REF), PSIMITags.REFTYPE_REF, null);
        sourceE.addContent (bibRefE);

        entryE.addContent(sourceE);

        return entryE;
    }

    protected void setNames (Element parent, String shortLabel, String fullName) {

        Element namesE = new Element (PSIMITags.NAMES, this.psins);
        if (shortLabel != null) {
            Element shortLabelE = new Element (PSIMITags.SHORT_LABEL, this.psins);
            shortLabelE.addContent (shortLabel);
            namesE.addContent (shortLabelE);
        } else {
            System.err.println ("No short label provided: " + parent.getName());
        }
        if (fullName != null && !fullName.equals (NO_VAL)) {
            Element fullNameE = new Element (PSIMITags.FULLNAME, this.psins);
            fullNameE.addContent (fullName);
            namesE.addContent (fullNameE);
        }
        if (namesE.getChildren().size() > 0) {
            parent.addContent (namesE);
        }
    }

    //potential helper method
    protected void setCVName (Element parent, String sourceTerm) {

        CVMapEntry CVMapped = CVMapEntry.getMappedEntry(sourceTerm);
        setNames (parent, CVMapped.getPSIMITerm_SL(), CVMapped.getPSIMITerm_FL());
    }

    //potential helper method
    protected void setCVRef (Element parent, String sourceTerm, String refType,
                             String refElementName, String refValue) {

        CVMapEntry CVMapped = CVMapEntry.getMappedEntry(sourceTerm);
        setXRef (parent, refElementName, CVMapped.getPSIMITerm_SL(),
                 CVMapped.getPSIMITermID (), refValue,
                 PSIMITags.getRefTypeAccn(refType), refType, null);
    }

    protected void setXRef (Element parent, String refElementName, String DBName, String
                          DBID) {

        setXRef (parent, refElementName, DBName, null, DBID, null, null, null);
    }

    protected void setXRef (Element parent, String refElementName, String DBName, String
                          DBAccn, String DBID, String refTypeAccn, String refType, String version) {

        Element xrefE = parent.getChild (PSIMITags.XREF, this.psins);
        if (xrefE == null) {
            xrefE = new Element (PSIMITags.XREF, this.psins);
            parent.addContent (xrefE);
        }
        setRef (xrefE, refElementName, DBName, DBAccn, DBID, refTypeAccn, refType,version);

        //return xrefE;
    }

     protected void setXRef (Element parent, String refElementName, String DBName, String
                          DBAccn, String DBID, String refTypeAccn, String refType, String attributeType, String attribute, String version) {

        Element xrefE = parent.getChild (PSIMITags.XREF, this.psins);
        if (xrefE == null) {
            xrefE = new Element (PSIMITags.XREF, this.psins);
            parent.addContent (xrefE);
        }


        Element current = setRef (xrefE, refElementName, DBName, DBAccn, DBID, refTypeAccn, refType, version);

        if(current != null)
            //add the type of mapped ref as an attribute for the secondary ref.
            addAttribute(current,attributeType, attribute );

    }


    protected Element setRef (Element parent, String name, String DBName, String
                         DBAccn, String DBID, String refTypeAccn, String refType, String version) {

       //check for redundancies
       ArrayList <Element> tempList = new ArrayList <Element> ();
       tempList.addAll (parent.getChildren());
       boolean found = false;
       for (int i = 0; i < tempList.size (); i++) {
           if (tempList.get (i).getAttributeValue(PSIMITags.DB_ATT).equals (DBName) &&
               tempList.get (i).getAttributeValue(PSIMITags.ID_ATT).equals (DBID)) {
               found = true;
               break;
           }
       }
       if (!found) {
           Element refE = new Element (name, this.psins);
           refE.setAttribute(PSIMITags.DB_ATT, DBName);
           if (DBAccn != null) {
               refE.setAttribute(PSIMITags.DBACC_ATT, DBAccn);
           }
           refE.setAttribute (PSIMITags.ID_ATT, DBID);
           if(version != null && !version.equalsIgnoreCase("")){
               refE.setAttribute(PSIMITags.XREF_VERSION,version);
           }
           if (refTypeAccn != null) {
               refE.setAttribute (PSIMITags.REFTYPEAC_ATT, refTypeAccn);
           }
           if (refType != null) {
               refE.setAttribute (PSIMITags.REFTYPE_ATT, refType);
           }


           parent.addContent (refE);
           return refE;
       }
        return null;
    }

    private void translateAllInteractions (Element entrySetE, Element BINDSubmitE)
                                       throws Exception {


        //Index of all the interactions in the current file
        //HashMap<String, Element> interaction_index = new HashMap<String, Element>();
         //intialize the interaction tracker so we can find interactions when creating complexes
       //scan the complexes and get the ids we need to store
       //BINDToPSIMITransComplex BINDComTrans = new BINDToPSIMITransComplex (inputFilesPath, fileName, psins,
         //                this.counter, bw, missing_pmids, Novel_GO_terms, error_log,stats_log,new_pmids,
         //               interaction_index,this, GI_index);
       //BINDComTrans.scanComplexes (interactionid_incomplexes,BINDSubmitE);

        Element entryE = entrySetE.getChild(PSIMITags.ENTRY, psins);
        if (entryE == null) {
            entryE = setEntry();
            entrySetE.addContent (entryE);
        }

        Element interactionListE = entryE.getChild (PSIMITags.INT_LIST, this.psins);
        if (interactionListE == null) {
            interactionListE = new Element (PSIMITags.INT_LIST, this.psins);
            entryE.addContent (interactionListE);
        }
        ArrayList <Element> bindIntList = XMLUtil.getElement (BINDTags.INTERACTION,
                                           BINDSubmitE, 1);
        for (int i = 0; i < bindIntList.size (); i++) {
            translateInteraction (interactionListE, bindIntList.get (i));

            //for each interaction create an index of the BIND ID to the interaction reference so
            //we can refer to it when converting the complexes.
            // ArrayList <Element> eList = XMLUtil.getElement (BINDTags.INTERACTION_ID, bindIntList.get (i),
              //                      XMLUtil.getDepth (BINDTags.INTERACTION));
            //BINDInteractionID = null;
            //if (XMLUtil.validateSingle (BINDTags.INTERACTION_ID, eList) &&
            //    XMLUtil.validateInt(eList.get (0))) {
            //    BINDInteractionID = eList.get (0).getTextTrim ();
            //}

            //add id to the unique set of  BIND interaction ids
            //bindInteractionId_index.put(BINDInteractionID, BINDInteractionID);

            //insert intearction ID and pointer into the hash map
            // //only put in HASH if it is in the list of IDs in complexes
            // --> replaced with pre-computed interaction subsets.
            //if(!interactionid_incomplexes.contains(BINDInteractionID))
                //interaction_index.put(BINDInteractionID,bindIntList.get (i) );
        }
        System.out.println ("For file: " + getInputFilesPath() + ": ");
        System.out.println ("Total Interactions: " + this.cntTotalInteraction +
                            " Translated: " + this.cntTransInteraction +
                            " Skipped: " + this.cntNonTransInteraction +
                            " Bad: " + this.cntBadInteraction +
                            " Int A - Protein: " + this.cntProteinA +
                            " Int B - Protein: " + this.cntProteinB);



        stats_log.write (this.currentTaxID);
        stats_log.write ("\tTotal Interactions: " + this.cntTotalInteraction +
                            " \tTranslated: " + this.cntTransInteraction +
                            " \tSkipped: " + this.cntNonTransInteraction +
                            " \tBad: " + this.cntBadInteraction +
                            " \tInt A - Protein: " + this.cntProteinA +
                            " \tInt B - Protein: " + this.cntProteinB );

        if (isTransComplexes()) {
            if (isTransProsOnly()) {
                System.err.println ("No support for complexes with 'protein only' option");
                return;
            }

            //check to see if this file has complexes
            ArrayList <Element> complexes = XMLUtil.getElement (BINDTags.COMPLEX,
                    BINDSubmitE, 1);
            if(complexes.size()>0){

                //get the interaction index from the precomputed subset of interactions needed for complex construction
                HashMap<String, Element> interaction_index = new HashMap<String, Element>();

                //open associated complex file
                File supplementary_file = new File (COMPLEX_PATH_PROP, "taxid" + this.currentTaxID + ".interactions_for_complexes.xml");

                //read in XML
                Element rootElement = XMLUtil.readSchemaBasedXML(supplementary_file.getAbsolutePath());

                //file data structure
                ArrayList <Element> intList = XMLUtil.getElement (BINDTags.INTERACTION,
                                           BINDSubmitE, 1);
                for (int i = 0; i < intList.size (); i++) {

                    //for each interaction create an index of the BIND ID to the interaction reference so
                    //we can refer to it when converting the complexes.
                    ArrayList <Element> eList = XMLUtil.getElement (BINDTags.INTERACTION_ID, intList.get (i),
                                    XMLUtil.getDepth (BINDTags.INTERACTION));
                    String ID = null;
                    if (XMLUtil.validateSingle (BINDTags.INTERACTION_ID, eList) &&
                        XMLUtil.validateInt(eList.get (0))) {
                        ID = eList.get (0).getTextTrim ();
                    }

                    //make sure the interaction is already in the set of translated interactions
                    //if the id is not in the set of translated interactions, translate it.
                    if((interactionsMap.get(ID) == null) && (badInteractionsMap.get(ID) == null)){
                        translateInteraction (interactionListE,intList.get(i));
                    }

                    interaction_index.put(ID,intList.get (i) );
                }

                //translate complexes, as PSI-MI interactions
                BINDToPSIMITransComplex BINDComTrans_translate =
                    new BINDToPSIMITransComplex (inputFilesPath, fileName, psins,
                         this.counter,bw, missing_pmids, Novel_GO_terms, error_log,stats_log,new_pmids,
                        interaction_index,this,GI_index, GI_history, interactionsMap,badInteractionsMap);
                BINDComTrans_translate.translateAllComplexes (entryE, BINDSubmitE);
            }
        }

        System.out.println(interactionTypeCount.toString());
    }

    //debugging
    private void listAllInteractions () {

        System.out.println ("Listing IDs for all interactions: BIND ID - PSI-MI ID");
        Iterator iterator = interactionsMap.keySet().iterator();
        while (iterator.hasNext ()) {
            String intID = (String)iterator.next ();
            System.out.println ("Int: " + intID + " " + interactionsMap.get (intID));
        }
        System.out.println ("Listing IDs for 'bad' interactions: BIND ID - PSI-MI ID");
        iterator = badInteractionsMap.keySet().iterator();
        while (iterator.hasNext ()) {
            String intID = (String)iterator.next ();
            System.out.println ("Bad Int: " + intID + " " + badInteractionsMap.get (intID));
        }
    }

    private void translateInteraction (Element interactionListE,
                                       Element BINDInteractionE) throws JDOMException, IOException {

        //parse the BIND interaction ID
        Element interactionE = new Element (PSIMITags.INT, this.psins);
        ArrayList <Element> eList = XMLUtil.getElement (BINDTags.INTERACTION_ID, BINDInteractionE,
                                    XMLUtil.getDepth (BINDTags.INTERACTION));
        BINDInteractionID = null;
        if (XMLUtil.validateSingle (BINDTags.INTERACTION_ID, eList) &&
            XMLUtil.validateInt(eList.get (0))) {
            BINDInteractionID = eList.get (0).getTextTrim ();
        }

        //parse the interaction simple description --> put in PSI-MI shortlabel
        eList = XMLUtil.getElement (BINDTags.SIMPLE_DESC,
                   BINDInteractionE, XMLUtil.getDepth (BINDTags.INTERACTION));
        String shortLabel = NO_VAL;
        if (XMLUtil.validateSingle (BINDTags.SIMPLE_DESC, eList)) {
            shortLabel = getDefText (eList.get (0), NO_VAL);
        }
        if (!shortLabel.equals (NO_VAL)) {
            setNames (interactionE, shortLabel, shortLabel);
        } else {
            setNames (interactionE, shortLabel, null);
            //this.badInteractionData = true;
        }
        setCVRef (interactionE, PSIMITags.BIND_SL, PSIMITags.REFTYPE_IDENT,
                  PSIMITags.PREF, BINDInteractionID);

        //get the sources associated only with the interactions.
        ArrayList<String> interaction_pmids = new ArrayList<String>();
        Element dummyE = new Element ("dummyElement", this.psins);

        //should get pubmed from interaction source first
        translatePubMed(dummyE,BINDInteractionE.getChild (BINDTags.INTERACTION_SOURCE),interaction_pmids );

        //get the places from the interaction --> add the places of the interaction to each interactor
        //check to see if there are places associated with interaction
        ArrayList<Element> additional_compartments = new ArrayList<Element>();
        ArrayList <Element> descrList = XMLUtil.getElement (BINDTags.DESC,
                         BINDInteractionE, XMLUtil.getDepth (BINDTags.INTERACTION));
         if(XMLUtil.validateSingle(BINDTags.DESC,  descrList)){
            Element descE = descrList.get(0);

            //get the places
            ArrayList <Element> placesList = XMLUtil.getElement (BINDTags.PLACE,
                             descE, XMLUtil.getDepth (/*"BIND-descr_place"*/descE.getName()),true);

            if (XMLUtil.validateExists(BINDTags.PLACE, placesList)) {

                //go through each place and see what the BIND-gen-place is
                for(int k = 0; k< placesList.size();k++){

                    //create a new comparment
                    Element compartmentE = new Element(PSIMITags.COMPARTMENT, this.psins);

                    //translate place
                    compartmentE = translatePlace(compartmentE, placesList.get(k),"BIND-descr_place","BINDInteractionID: " +BINDInteractionID,interaction_pmids);

                    //add the compartment to the organism
                    if(compartmentE != null){
                        //add an attribute to the interactor indicating that the compartment was taken from the interaction and not the interactor.
                        addAttribute(compartmentE,PSIMITags.ATT_Int_place,"interaction place");

                        additional_compartments.add(compartmentE);
                    }
                } //for loop - for each place
            }
          }


        //need to add the experiment before the interactors (but we need to get the species info before we can translate
        //and create the experiments so add a place holder for the experiments before adding the interactors)
        Element expListE = new Element (PSIMITags.EXP_LIST, this.psins);
        interactionE.addContent (expListE);

        //create participant list to store the interactors.
        Element partListE = new Element (PSIMITags.PARTLIST, this.psins);
        interactionE.addContent (partListE);

        //1st BIND interactor - create a particpant object and an interactor object
        Element partE = new Element (PSIMITags.PART, this.psins);

        partE.setAttribute(PSIMITags.ID_ATT, Integer.toString (++this.counter));

        partListE.addContent (partE);
        Element interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
        partE.addContent (interactorE);

        //store a point for the interactor for the experimental form (if there is one)
        Element interactor_A = BINDInteractionE.getChild (BINDTags.INTERACT_A);

        int species_A = translateInteractor (interactorE, BINDInteractionE.getChild (BINDTags.INTERACT_A), ++this.counter,interaction_pmids,additional_compartments,null);

        //add the experimental role from Bind-condition_bait-condition
        //can have one of three values: a-is-bait, b-is-bait,not-applicable
        eList = XMLUtil.getElement(BINDTags.BAIT_COND, BINDInteractionE, XMLUtil.getDepth(BINDTags.INTERACTION));
        String bait_condition = "";
        if (XMLUtil.validateSingle (BINDTags.BAIT_COND, eList)) {
           bait_condition = eList.get(0).getAttributeValue("value");
        }

        if(bait_condition.equalsIgnoreCase("a-is-bait"))
                addExperimentalRole(partE,"bait");
        else if(bait_condition.equalsIgnoreCase("b-is-bait"))
                addExperimentalRole(partE,"prey");
        else if(bait_condition.equalsIgnoreCase("not-applicable"))
                addExperimentalRole(partE,"none");

        //check to see if there is an experimental form associated with this molecule
        eList = XMLUtil.getElement(BINDTags.EXPFORM_A, BINDInteractionE, XMLUtil.getDepth(BINDTags.INTERACTION));
        if (XMLUtil.validateSingle (BINDTags.EXPFORM_A, eList)) {
            //create experimental Interactor List to store experimental form
            Element experIntListE = new Element (PSIMITags.EXPERINTLIST, this.psins);

            Element experIntE = new Element(PSIMITags.EXPERINT, this.psins);
            experIntListE.addContent(experIntE);


            Element Form_interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
            experIntE.addContent(Form_interactorE);

            //get the experimenatl form
            //Element expform = eList.get(0).getChild(BINDTags.EXPFORM);

            //check to see if the experimental form is a BIND-object.  (an experimental form can also be
            //  a profile or a gene.  Only parse the experimental forms that are objects.
            ArrayList <Element> check_type = XMLUtil.getElement(BINDTags.EXPFORM_OBJECT, eList.get(0), XMLUtil.getDepth(BINDTags.EXPFORM_A));
            if (XMLUtil.validateSingle (BINDTags.EXPFORM_OBJECT, check_type)){
                //translate the experimental form
                translateInteractor (Form_interactorE, eList.get(0), ++this.counter,interaction_pmids, null, interactor_A);
                partE.addContent(experIntListE);
            }

        }


        //2nd BIND interactor - create a particpant object and an interactor object
        partE = new Element (PSIMITags.PART, this.psins);

        partE.setAttribute(PSIMITags.ID_ATT, Integer.toString (++this.counter));

        partListE.addContent (partE);
        interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
        partE.addContent (interactorE);

        Element interactor_B = BINDInteractionE.getChild(BINDTags.INTERACT_B);
        int species_B = translateInteractor (interactorE, BINDInteractionE.getChild(BINDTags.INTERACT_B), ++this.counter,interaction_pmids, additional_compartments,null);

        if(bait_condition.equalsIgnoreCase("a-is-bait"))
                addExperimentalRole(partE,"prey");
        else if(bait_condition.equalsIgnoreCase("b-is-bait"))
                addExperimentalRole(partE,"bait");
        else if(bait_condition.equalsIgnoreCase("not-applicable"))
                addExperimentalRole(partE,"none");

         //check to see if there is an experimental form associated with this molecule
        eList = XMLUtil.getElement(BINDTags.EXPFORM_B, BINDInteractionE, XMLUtil.getDepth(BINDTags.INTERACTION));
        if (XMLUtil.validateSingle (BINDTags.EXPFORM_B, eList)) {
            //create experimental Interactor List to store experimental form
            Element experIntListE = new Element (PSIMITags.EXPERINTLIST, this.psins);

            Element experIntE = new Element(PSIMITags.EXPERINT, this.psins);
            experIntListE.addContent(experIntE);

            Element Form_interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
            experIntE.addContent(Form_interactorE);

            //get the experimenatl form
            //Element expform = eList.get(0).getChild(BINDTags.EXPFORM);

            //translate the experimental form
            //check to see if the experimental form is a BIND-object.  (an experimental form can also be
            //  a profile or a gene.  Only parse the experimental forms that are objects.
            ArrayList <Element> check_type = XMLUtil.getElement(BINDTags.EXPFORM_OBJECT, eList.get(0), XMLUtil.getDepth(BINDTags.EXPFORM_B));
            if (XMLUtil.validateSingle (BINDTags.EXPFORM_OBJECT, check_type)){
                //translate the experimental form
                translateInteractor (Form_interactorE, eList.get(0), ++this.counter,interaction_pmids, null, interactor_B);
                partE.addContent(experIntListE);
            }

        }


        //keep track of the stats for the interctions
        countInteractionType(getBINDInteractorType(BINDInteractionE.getChild(BINDTags.INTERACT_A)), getBINDInteractorType(BINDInteractionE.getChild(BINDTags.INTERACT_B)));

        //check to see if species_A and species_B match
        //in order to transfer this info to the interaction_descr BIND-place they need to agree.
        //TODO : the host species described in the experiment description is not the same as the host system...set to zero.
        int speciesID;
        if(species_A != species_B)
            if(species_A == Integer.parseInt(PSIMITags.TAXID_UN_ID) )
                 speciesID = species_B;
            else if(species_B == Integer.parseInt(PSIMITags.TAXID_UN_ID))
                speciesID = species_A;
            //if they don't equal and neither one of them is the undefined species id use zero as the default species id.
            else
                speciesID = 0;
        else
            speciesID = species_A;

        speciesID = 0;

        //parse the experiment and translate experiment description
        ArrayList<String> experiment_pmids = translateExpDesc (expListE, BINDInteractionE, BINDInteractionID, speciesID, interaction_pmids);

        //extract unique information from the BIND-descr which contains binding place, binding sites, experimental forms
        eList = XMLUtil.getElement(BINDTags.DESC, BINDInteractionE, XMLUtil.getDepth(BINDTags.INTERACTION));
        if (XMLUtil.validateSingle (BINDTags.DESC, eList)) {
            translateDescr( interactionE,eList.get(0),BINDInteractionE);
        }

        eList = XMLUtil.getElement (BINDTags.INTERACTION_COL, BINDInteractionE,
                                    XMLUtil.getDepth (BINDTags.INTERACTION));
        if (XMLUtil.validateSingle (BINDTags.INTERACTION_COL, eList)) {
            String BINDDivision = BINDTags.checkBINDDiv (eList.get (0).getTextTrim ());
            addAttribute (interactionE, PSIMITags.ATT_BIND_DIVISION,
                          BINDDivision);
        }
        addInteraction (interactionListE, interactionE);
    }

    //interaction counter increments regardless. Flags control.
    private void addInteraction (Element interactionListE, Element interactionE)
                                                          throws IOException {
        Integer interactionID = ++this.counter;
        if (!this.badInteractionData) {
            interactionsMap.put (BINDInteractionID, String.valueOf (interactionID));
        }

        interactionE.setAttribute(PSIMITags.ID_ATT,
                                      Integer.toString (interactionID));

        if (!this.transProOnlyFlag || (this.proteinAFlag && this.proteinBFlag)) {
            this.cntTransInteraction++;
            if (!this.badInteractionData) {
                interactionListE.addContent (interactionE);
            } else {
                if (!this.cleanDataFlag) {                //not cleaning anyway
                    interactionListE.addContent (interactionE);
                }
                this.badInteractionData = false;
                addAttribute (interactionE, PSIMITags.ATT_BAD_DATA_FILE, fileName);
                XMLUtil.dump(interactionE, bw);
                bw.newLine();
                this.cntBadInteraction++;
                badInteractionsMap.put (BINDInteractionID,
                   interactionE.getAttributeValue (PSIMITags.ID_ATT));
            }
        } else {
            this.cntNonTransInteraction++;
            if (this.badInteractionData == true) {
                this.badInteractionData = false;
            }
        }
        this.cntTotalInteraction++;

        //track proteins anyway
        if (this.proteinAFlag) {
            this.cntProteinA++;
        }
        if (this.proteinBFlag) {
            this.cntProteinB++;
        }
        this.proteinAFlag = false;
        this.proteinBFlag = false;
    }
    /*
    * Mehtod to translate Experimental description
    *
    * added the species ID to parameters because the host organism that is used to store the compartment requires a species ID
    *
    * @return - the list of publications associated with this experimental description
    */
    protected ArrayList<String> translateExpDesc (Element expListE, Element BINDInteractionE,
                                                  String BINDInteractionID, int speciesID, ArrayList<String> interaction_pmids) throws JDOMException, IOException {

        //create a place holder for all the pubmeds associated with this experiment
        ArrayList <String> experiment_Pubmedids = new ArrayList<String>();

        //get the bind condition element
        ArrayList <Element> eList = XMLUtil.getElement (BINDTags.COND,
                         BINDInteractionE, XMLUtil.getDepth (BINDTags.INTERACTION));

        //no BIND-condition (no interaction detection method)
        if (!XMLUtil.validateExists (BINDTags.COND, eList)) {     //no exp condition

             //create the experiment description object
            Element expDescE = new Element (PSIMITags.EXP_DESC, this.psins);

            expDescE.setAttribute(PSIMITags.ID_ATT, Integer.toString(++this.counter));
            expListE.addContent (expDescE);

            Element intDetMethodE = new Element (PSIMITags.INT_DET_METHOD, this.psins);

            //un-specified method
            setNames(intDetMethodE, PSIMITags.SL_METHOD_NS, PSIMITags.SL_METHOD_NS);
            setXRef(intDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                   PSIMITags.ID_METHOD_NS, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);

            //if there is no experiment condition the publications have to come from the interaction source
            translatePubMed(expDescE,
                      BINDInteractionE.getChild (BINDTags.INTERACTION_SOURCE),experiment_Pubmedids);

            expDescE.addContent (intDetMethodE);

             //for each experiment we need a participant detection method but this
             //has yet to be translated from BIND
             //TODO: translate participant identifications
             Element partiDetMethodE = new Element (PSIMITags.PARTI_DET_METHOD, this.psins);
             //inferred by curator method
             setNames(partiDetMethodE, PSIMITags.SL_METHOD_IC, PSIMITags.SL_METHOD_NS);
             setXRef(partiDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                   PSIMITags.ID_METHOD_IC, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT, null);
             expDescE.addContent (partiDetMethodE);


            error_log.write("BINDInteraction ID:" + BINDInteractionID + "\t No exp. conditions \n");
            //System.err.println ("No exp. conditions " + expID);
        }
        //go through each of the BIND-conditions.
        else {
            for (int i = 0; i < eList.size (); i++) {

                 //create the experiment description object
                Element expDescE = new Element (PSIMITags.EXP_DESC, this.psins);

                expDescE.setAttribute(PSIMITags.ID_ATT, Integer.toString(++this.counter));

                expListE.addContent (expDescE);

                //create a current experimentLIst
                ArrayList <String> current_experiment_Pubmedids = new ArrayList<String>();

                Element intDetMethodE = new Element (PSIMITags.INT_DET_METHOD, this.psins);
                boolean isIntDetMethod = translateDetectionMethod (expDescE, intDetMethodE, eList.get (i));
                if (intDetMethodE.getChildren().size() == 0) {
                    setNames(intDetMethodE, PSIMITags.SL_METHOD_NS, PSIMITags.SL_METHOD_NS);
                    setXRef(intDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                            PSIMITags.ID_METHOD_NS, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                }
                //check to see if it is a participant detection method.
                //for each experiment we need a participant detection method but this
                //has yet to be translated from BIND
                //TODO: translate participant descriptions
                 Element partiDetMethodE = new Element (PSIMITags.PARTI_DET_METHOD, this.psins);
                if(!isIntDetMethod){
                      boolean isPartiMethod = translatePartiDetectionMethod (expDescE, partiDetMethodE, eList.get (i));

                    //must be a feature detection method
                    //TODO: translate feature detection methods.
                    if(!isPartiMethod){
                        //inferred by curator method
                        setNames(partiDetMethodE, PSIMITags.SL_METHOD_IC, PSIMITags.SL_METHOD_NS);
                        setXRef(partiDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                        PSIMITags.ID_METHOD_IC, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                    }
                }
                else{
                    //inferred by curator method
                    setNames(partiDetMethodE, PSIMITags.SL_METHOD_IC, PSIMITags.SL_METHOD_NS);
                    setXRef(partiDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                    PSIMITags.ID_METHOD_IC, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                }
                //should get pubmed from interaction source first
                translatePubMed(expDescE,BINDInteractionE.getChild (BINDTags.INTERACTION_SOURCE),current_experiment_Pubmedids);

                //get the pubmeds associated with each Condition
                translatePubMed (expDescE, eList.get (i),current_experiment_Pubmedids); //order is signifcant

                // TODO: which publications are only found in the experiment publcations.
                 // compare the experiment and  interaction publications.

                 //check to see if the experiment pmids is in the interaction pmid
/*                for(int l = 0; l < current_experiment_Pubmedids.size();l++){
                    boolean found = false;
                    for(int m = 0 ;m<interaction_pmids.size();m++){
                       if(current_experiment_Pubmedids.get(l).equalsIgnoreCase(interaction_pmids.get(m)))
                         found = true;
                    }
                    if(!found && !interaction_pmids.isEmpty() && !current_experiment_Pubmedids.isEmpty())
                        System.out.println("condition pmid not in interaction pmids: " + current_experiment_Pubmedids.get(l) +  "( interaction pubs: "+ interaction_pmids.toString() + ")");
                }*/

                //add these pubmeds to the experiment pubmed ids
                experiment_Pubmedids.addAll(current_experiment_Pubmedids);


                //If there is no publication associated with this record use the BIND ID
                //TODO: fix this.  There should always be a publication associated with a record!
                if (expDescE.getChildren(PSIMITags.BIB, this.psins).size() == 0) {

                    error_log.write("BINDInteraction ID:" + BINDInteractionID + "\t No pmids associated with the interaction \n");

                    //bib ref can be either a pubmed or an attribute list
                    //associate the bind id and the title of the general citation
                    //in an attribute list

                    Element bibRefE = new Element (PSIMITags.BIB, this.psins);

                    //requires an xRef for a bibref. Will crash if missing.
                    //add dummy pubmid reference

                    //don't need the dummy pubmed anymore.
                    //CVMapEntry CVMapper = CVMapEntry.getMappedEntry (PSIMITags.PUBMED_SL);
                    //setXRef (bibRefE, PSIMITags.PREF, CVMapper.getPSIMITerm_SL(),
                        //CVMapper.getPSIMITermID (),
                        //PSIMITags.NO_PUBMED,
                        //PSIMITags.getRefTypeAccn(PSIMITags.REFTYPE_REF),
                        //PSIMITags.REFTYPE_REF);

                    //check to see if we can get additional info from general publication record
                    ArrayList <Element> PubGenList = XMLUtil.getElement (BINDTags.PUB_GEN,
                            BINDInteractionE, XMLUtil.getDepth (BINDInteractionE.getName()));
                    if (XMLUtil.validateSingle(BINDTags.PUB_GEN,PubGenList)){
                        for(int k=0; k<PubGenList.size();k++){
                            //get the general citation
                            Element gen_cit = PubGenList.get(k).getChild(BINDTags.CIT_GEN);
                            if(gen_cit != null){
                                String title = gen_cit.getChildText(BINDTags.CIT_GEN_TITLE);
                                String cit = gen_cit.getChildText(BINDTags.CIT_GEN_CIT);

                                //check the title to see if we have mapped the title to a pmid
                                if(new_pmids.getPMIDMap().get(title) != null){
                                    CVMapEntry CVMapper = CVMapEntry.getMappedEntry (PSIMITags.PUBMED_SL);

                                    setXRef (bibRefE, PSIMITags.PREF, CVMapper.getPSIMITerm_SL(),
                                        CVMapper.getPSIMITermID (),
                                        new_pmids.getPMIDMap().get(title),
                                        PSIMITags.getRefTypeAccn(PSIMITags.REFTYPE_REF),
                                        PSIMITags.REFTYPE_REF,null);

                                    //addAttribute(bibRefE,PSIMITags.ATT_GEN_PUB_TITLE,title);
                                }
                                else{
                                    //write out the publcation that is missing
                                    missing_pmids.write("BINDInteractionID:" + BINDInteractionID + "\t" + title + "\t" + cit + "\n");
                                    addAttribute(bibRefE,PSIMITags.ATT_BIND_Record,BINDInteractionID);
                                    addAttribute(bibRefE,PSIMITags.ATT_GEN_PUB_TITLE,title);
                                }
                            }
                        }
                    }
                    else
                        addAttribute(bibRefE,PSIMITags.ATT_BIND_Record,BINDInteractionID);
                    expDescE.addContent(bibRefE);

                }

                //order matters
                //add the interaction detection method after the above information
                //names, bibref,xref, hostorganism(places),interaction detection, participant identifaction, confidence, attribute

                expDescE.addContent (intDetMethodE);

                expDescE.addContent (partiDetMethodE);


            }//for loop - for each condition

        }//else - there are conditions
        return experiment_Pubmedids;
    }

    private boolean translateDetectionMethod (Element expDescE, Element intDetMethodE,
                                              Element BINDConditionE) throws JDOMException, IOException {
        boolean isIntDetectionMethod = true;
        String shortLabel = null;
        String fullName = null;

        ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.COND_DESC,
                BINDConditionE, XMLUtil.getDepth (BINDTags.COND));
        if (XMLUtil.validateSingle(BINDTags.COND_DESC, tempList)) {
            fullName = getDefText(tempList.get (0), NO_VAL);
        }
        tempList = XMLUtil.getElement (BINDTags.EXP_SYS, BINDConditionE,
                               XMLUtil.getDepth (BINDTags.COND));
        if (XMLUtil.validateSingle (BINDTags.EXP_SYS, tempList)) {
            shortLabel = tempList.get (0).getAttributeValue(BINDTags.VALUE_ATT);
            CVMapEntry CVMapped = CVMapEntry.getMappedEntry(shortLabel);
            //temp section, in case more BIND non-consistent exp. systems
            if (CVMapped == null) {
                error_log.write("Error: Undetected BIND exp: " + shortLabel + "\n");
                System.err.println ("Error: Undetected BIND exp: " + shortLabel);
                setNames (intDetMethodE, shortLabel, shortLabel);
                setXRef(intDetMethodE, PSIMITags.PREF, PSIMITags.BIND_SL, PSIMITags.ID_BIND,
                        tempList.get (0).getTextTrim(), null, null,null);
            } else {
            //end temp section
                //check to see the type of CVTerm
                if(CVMapped.getTargetCategory().equalsIgnoreCase("interactionDetectionMethod")){

                    setNames (intDetMethodE, CVMapped.getPSIMITerm_SL(),
                          CVMapped.getPSIMITerm_FL());
                    if (CVMapped.getPSIMITermID () != null) {
                        setXRef(intDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                            CVMapped.getPSIMITermID (), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                    } else {
                        setXRef(intDetMethodE, PSIMITags.PREF, PSIMITags.BIND_SL, PSIMITags.ID_BIND,
                            CVMapped.getSourceID(), null, null,null);
                    }
                    setNames(expDescE, shortLabel, fullName);
                }else
                    isIntDetectionMethod = false;
            }
        }

        return isIntDetectionMethod;
    }

     private boolean translatePartiDetectionMethod (Element expDescE, Element partiDetMethodE,
                                                    Element BINDConditionE) throws JDOMException, IOException {
        boolean isPartiDetectionMethod = true;
        String shortLabel = null;
        String fullName = null;

        ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.COND_DESC,
                BINDConditionE, XMLUtil.getDepth (BINDTags.COND));
        if (XMLUtil.validateSingle(BINDTags.COND_DESC, tempList)) {
            fullName = getDefText(tempList.get (0), NO_VAL);
        }
        tempList = XMLUtil.getElement (BINDTags.EXP_SYS, BINDConditionE,
                               XMLUtil.getDepth (BINDTags.COND));
        if (XMLUtil.validateSingle (BINDTags.EXP_SYS, tempList)) {
            shortLabel = tempList.get (0).getAttributeValue(BINDTags.VALUE_ATT);
            CVMapEntry CVMapped = CVMapEntry.getMappedEntry(shortLabel);
            //temp section, in case more BIND non-consistent exp. systems
            if (CVMapped == null) {
                error_log.write("Error: Undetected BIND exp: " + shortLabel + "\n");
                System.err.println ("Error: Undetected BIND exp: " + shortLabel);
                setNames (partiDetMethodE, shortLabel, shortLabel);
                setXRef(partiDetMethodE, PSIMITags.PREF, PSIMITags.BIND_SL, PSIMITags.ID_BIND,
                        tempList.get (0).getTextTrim(), null, null,null);
            } else {
            //end temp section
                //check to see the type of CVTerm
                if(CVMapped.getTargetCategory().equalsIgnoreCase("participantDetectionMethod")){

                    setNames (partiDetMethodE, CVMapped.getPSIMITerm_SL(),
                          CVMapped.getPSIMITerm_FL());
                    if (CVMapped.getPSIMITermID () != null) {
                        setXRef(partiDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                            CVMapped.getPSIMITermID (), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                    } else {
                        setXRef(partiDetMethodE, PSIMITags.PREF, PSIMITags.BIND_SL, PSIMITags.ID_BIND,
                            CVMapped.getSourceID(), null, null,null);

                    }
                    setNames(expDescE, shortLabel, fullName);
                }else
                    isPartiDetectionMethod = false;
            }
        }

        return isPartiDetectionMethod;
    }

    protected ArrayList <String> translatePubMed (Element PSIReferencedE, Element
                                  BINDReferencedE, ArrayList<String> addedPubmedids, ArrayList<String> uniqueParents, boolean search_medline) throws JDOMException {


        //there are four different place that pubmedid can be - check all places
        //PubMedId
        ArrayList <Element> pubMedList = XMLUtil.getElement (BINDTags.PUBMED_ID,
           BINDReferencedE,uniqueParents, XMLUtil.getDepth (BINDReferencedE.getName(), uniqueParents));
        if(XMLUtil.validateExists(BINDTags.PUBMED_ID,pubMedList))
            translateReferences (pubMedList, PSIReferencedE,addedPubmedids);
        //Pub_pmid
        pubMedList = XMLUtil.getElement (BINDTags.PUBMED_PMID,
           BINDReferencedE,uniqueParents, XMLUtil.getDepth (BINDReferencedE.getName(), uniqueParents));
        if(XMLUtil.validateExists(BINDTags.PUBMED_PMID,pubMedList))
            translateReferences (pubMedList, PSIReferencedE,addedPubmedids);


        //Pub_medline
        if(search_medline){
            pubMedList = XMLUtil.getElement (BINDTags.PUBMEDMEDLINE_ID,
                BINDReferencedE, uniqueParents,XMLUtil.getDepth (BINDReferencedE.getName(), uniqueParents));
            if(XMLUtil.validateExists(BINDTags.PUBMEDMEDLINE_ID,pubMedList))
                translateReferences (pubMedList, PSIReferencedE,addedPubmedids);
        }
        //Pub_muid

       pubMedList = XMLUtil.getElement (BINDTags.PUBMEDMU_ID,
                BINDReferencedE,uniqueParents, XMLUtil.getDepth (BINDReferencedE.getName(), uniqueParents));
       translateReferences (pubMedList, PSIReferencedE,addedPubmedids);

        return addedPubmedids;
    }

    protected ArrayList <String> translatePubMed (Element PSIReferencedE, Element
                                  BINDReferencedE, ArrayList<String> addedPubmedids) throws JDOMException {


        //there are four different place that pubmedid can be - check all places
        //PubMedId
        ArrayList <Element> pubMedList = XMLUtil.getElement (BINDTags.PUBMED_ID,
           BINDReferencedE, XMLUtil.getDepth (BINDReferencedE.getName()), true);
        if(XMLUtil.validateExists(BINDTags.PUBMED_ID,pubMedList))
            translateReferences (pubMedList, PSIReferencedE,addedPubmedids);
        //Pub_pmid
        pubMedList = XMLUtil.getElement (BINDTags.PUBMED_PMID,
           BINDReferencedE, XMLUtil.getDepth (BINDReferencedE.getName()), true);
        if(XMLUtil.validateExists(BINDTags.PUBMED_PMID,pubMedList))
            translateReferences (pubMedList, PSIReferencedE,addedPubmedids);


        //Pub_medline
        pubMedList = XMLUtil.getElement (BINDTags.PUBMEDMEDLINE_ID,
           BINDReferencedE, XMLUtil.getDepth (BINDReferencedE.getName()), true);
        if(XMLUtil.validateExists(BINDTags.PUBMEDMEDLINE_ID,pubMedList))
            translateReferences (pubMedList, PSIReferencedE,addedPubmedids);

        //Pub_muid
        //if(search_muid){
            pubMedList = XMLUtil.getElement (BINDTags.PUBMEDMU_ID,
                BINDReferencedE, XMLUtil.getDepth (BINDReferencedE.getName()), true);
            translateReferences (pubMedList, PSIReferencedE,addedPubmedids);
        //}
        return addedPubmedids;
    }

    //helper method
    //Only add the publication if it hasn't already been added to the set of publications.
    protected ArrayList <String> translateReferences (ArrayList <Element> pubMedList, Element
            PSIReferencedE, ArrayList<String> addedPubmedids) throws JDOMException {

        boolean addFlag = false;
        boolean firstTime = true;
        //ArrayList <String> pubmedids =new ArrayList<String>();
        CVMapEntry CVMapper = CVMapEntry.getMappedEntry (PSIMITags.PUBMED_SL);

        Element bibRefE = PSIReferencedE.getChild (PSIMITags.BIB, this.psins);
        if (bibRefE == null) {
            bibRefE = new Element (PSIMITags.BIB, this.psins);
            addFlag = true;
        } else {
            firstTime = false;
        }

        for (int j = 0; j < pubMedList.size (); j++) {
            String pubmedid = getDefText(pubMedList.get (j), NO_VAL);
            boolean add = true;
            //check to see if this pubmed has already been added
            for(int k= 0; k< addedPubmedids.size();k++)
                if(addedPubmedids.get(k).equalsIgnoreCase(pubmedid))
                    add = false;
            if(add && !pubmedid.equalsIgnoreCase(NO_VAL)){
                addedPubmedids.add(pubmedid);
                if (firstTime) {
                    setXRef (bibRefE, PSIMITags.PREF, CVMapper.getPSIMITerm_SL(),
                        CVMapper.getPSIMITermID (),
                        pubmedid,
                        PSIMITags.getRefTypeAccn(PSIMITags.REFTYPE_REF),
                        PSIMITags.REFTYPE_REF,null);
                    firstTime = false;
                } else {
                    setXRef (bibRefE, PSIMITags.SREF, CVMapper.getPSIMITerm_SL(),
                        CVMapper.getPSIMITermID (),
                        pubmedid,
                        PSIMITags.getRefTypeAccn(PSIMITags.REFTYPE_REF),
                        PSIMITags.REFTYPE_REF,null);
                }
            }//add loop
        } //for loop
        if (bibRefE.getChildren().size() > 0 && addFlag) {
            PSIReferencedE.addContent (bibRefE);
        }
        return addedPubmedids;
    }

   /* Parse an individual interactor from the given BIND XML
   *
   * @param interactorE - the PSIMI XML pointer to store the newly created interactor
   * @param BINDInteractorE - the BIND XML interactor to get all the data related to this interactor
   * @param interatorID - unique number used for the interaction ID
   * @param interaction_pmids - pmids currently associated with this interaction
   */
   protected int translateInteractor (Element interactorE, Element
                   BINDInteractorE, int interactorID, ArrayList<String> interaction_pmids,
                                    ArrayList<Element> additional_compartments, Element main_interactor) throws JDOMException, IOException {

       boolean is_expform = false;

        interactorE.setAttribute(PSIMITags.ID_ATT, Integer.toString (interactorID));

        Element interactorTypeE = new Element (PSIMITags.INT_TYPE, this.psins);
        Element BINDInteractorTypeE = translateInteractorType (interactorTypeE, BINDInteractorE);

       if(BINDInteractorTypeE != null){
            String BINDInteractorType = BINDInteractorTypeE.getName();
            //if the interactor Type is unspecified, check to see if this is an experimental form
            //if it is an experimental form grab the type from the main object.
            if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_NS) && (main_interactor != null)) {
                interactorTypeE = new Element (PSIMITags.INT_TYPE, this.psins);
                BINDInteractorTypeE = translateInteractorType (interactorTypeE,main_interactor);
                is_expform = true;
            }
       }
       else if(main_interactor != null){ //if it is an experimental form but doesn't even have an interactor type defined.
            interactorTypeE = new Element (PSIMITags.INT_TYPE, this.psins);
            BINDInteractorTypeE = translateInteractorType (interactorTypeE,main_interactor);
            is_expform = true;
       }
        String shortLabel = NO_VAL;
        String fullName = null;
        Element BINDNameE = null;
        ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.OBJ_SHORT_LABEL,
                        BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));

       //Get the BIND-object short label
       if (XMLUtil.validateSingle(BINDTags.OBJ_SHORT_LABEL, tempList)) {
            shortLabel = getDefText(tempList.get (0), NO_VAL);
            BINDNameE = tempList.get (0);
        }
        if (shortLabel.equals (BINDToPSIMITrans.NO_VAL)) {
            //this.badInteractionData = true;
        }

       //Get the BIND-object desc
        tempList = XMLUtil.getElement (BINDTags.OBJ_DESC, BINDInteractorE,
                               XMLUtil.getDepth(BINDInteractorE.getName ()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_DESC, tempList)) {
            fullName = getDefText(tempList.get (0), NO_VAL);
        }
        //order of addition is significant.
        setNames (interactorE, shortLabel, fullName);
        tempList = XMLUtil.getElement (BINDTags.OBJ_EXTREF,
                BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        Element BINDExtRefE = null;
        if (XMLUtil.validateSingle(BINDTags.OBJ_EXTREF, tempList)) {
            BINDExtRefE = tempList.get (0);
        }


        //get organism info
        Integer interactorSpeciesID = -1;
        Element organismE = new Element (PSIMITags.ORG, this.psins);



       //TODO:Need to transfer organism info from main interactor for experimental forms if nothing is specified in experimental form
        //Get the Object_id_id in the Biosource record which is the taxid.
        tempList = XMLUtil.getElement (BINDTags.OBJECT_ID_ID, BINDInteractorE,
                   XMLUtil.getDepth (BINDInteractorE.getName()), false);
        ArrayList <Element> orgNameList = XMLUtil.getElement (BINDTags.ORG_TAXNAME,
                      BINDInteractorE, XMLUtil.getDepth (BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJECT_ID_ID, tempList) &&
            XMLUtil.validateInt(tempList.get (0))) {
            organismE.setAttribute(PSIMITags.NCBI_TAXID,
                                   getDefText(tempList.get (0), NO_VAL));
            interactorSpeciesID = Integer.parseInt (getDefText(tempList.get (0), NO_VAL));

        }

       //Check to see if there is any cellular place for this molecule
       ArrayList <Element> placesList;

       if(BINDInteractorE.getName().equalsIgnoreCase(BINDTags.EXPFORM_A) ||
               BINDInteractorE.getName().equalsIgnoreCase(BINDTags.EXPFORM_B))
           placesList = XMLUtil.getElement (BINDTags.PLACE,
                 BINDInteractorE, XMLUtil.getDepth (BINDInteractorE.getName()),false);
       else
            placesList = XMLUtil.getElement (BINDTags.PLACE,
                             BINDInteractorE, XMLUtil.getDepth (BINDInteractorE.getName()),true);

       boolean location_added = false;

       if (XMLUtil.validateExists(BINDTags.PLACE, placesList)) {
            //go through each place and see what the BIND-gen-place is
            for(int i = 0; i< placesList.size();i++){

                //create a new comparment
                Element compartmentE = new Element(PSIMITags.COMPARTMENT, this.psins);

                //translate place
                compartmentE = translatePlace(compartmentE, placesList.get(i), BINDInteractorE.getName(),"Interactor short label"+shortLabel,interaction_pmids);

                //add the compartment to the organism
                if(compartmentE != null && !location_added){
                    organismE.addContent(compartmentE);
                    location_added = true;
                }
                else if(compartmentE != null && location_added)
                    error_log.write("Compartment already added, only place for one compartment. Interactor id:" +interactorID + " compartment:" + compartmentE.getValue().toString() + "not Added.");

            }
        }

       //check to see if there are any compartments that came from the interaction
       if(additional_compartments != null && !additional_compartments.isEmpty()){
           for(int l = 0; l< additional_compartments.size();l++){
               Element compartmentInteraction = (Element)additional_compartments.get(l).clone();
               if(compartmentInteraction != null){
                   if(compartmentInteraction != null && !location_added){
                        organismE.addContent(compartmentInteraction);
                       location_added = true;
                   }
                   else if(compartmentInteraction != null && location_added)
                        error_log.write("Compartment already added, only place for one compartment. Interactor id:" +interactorID + " compartment:" + compartmentInteraction.getValue().toString() + "not Added.");

               }

           }
       }

        if (interactorSpeciesID == -1) {
            //System.err.println ("Missing org info for: " + interactorID);
            interactorSpeciesID = Integer.parseInt (PSIMITags.TAXID_UN_ID);
        }
        addSpecies(interactorSpeciesID);
       Integer gi_taxid;
       //if this is an experimental form then there is no id, get it from the parent molecule.
       if(is_expform)
            gi_taxid = translateInteractorID (interactorE, main_interactor, BINDExtRefE,
                 BINDNameE, BINDInteractorTypeE, interactorSpeciesID);
       else
            gi_taxid = translateInteractorID (interactorE, BINDInteractorE, BINDExtRefE,
                 BINDNameE, BINDInteractorTypeE, interactorSpeciesID);    //forces null exception
        interactorE.addContent (interactorTypeE);

        tempList = XMLUtil.getElement (BINDTags.OBJECT_ID_ID, BINDInteractorE,
                   XMLUtil.getDepth (BINDInteractorE.getName()), false);
        orgNameList = XMLUtil.getElement (BINDTags.ORG_TAXNAME,
                      BINDInteractorE, XMLUtil.getDepth (BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJECT_ID_ID, tempList) &&
            XMLUtil.validateInt(tempList.get (0))) {


            //if the gi_taxid and the taxid in the record are differnt then use the one from the gi
            if(interactorSpeciesID != gi_taxid)
                organismE.setAttribute(PSIMITags.NCBI_TAXID,gi_taxid.toString());
            else
                 organismE.setAttribute(PSIMITags.NCBI_TAXID,interactorSpeciesID.toString());


            if (XMLUtil.validateSingle (BINDTags.ORG_TAXNAME, orgNameList)) {
                String orgName = getDefText(orgNameList.get (0), NO_VAL);
                if (!orgName.equals (NO_VAL)) {
                    setNames (organismE, orgName, orgName);
                } else {
                    System.err.println ("Bad org name: " +
                                        orgNameList.get (0).getTextTrim());
                }
            } else {
                System.err.println ("Error: tax ID but no org name in BIND.");
            }
        } else {                                              //unidentified
            organismE.setAttribute (PSIMITags.NCBI_TAXID, PSIMITags.TAXID_UN_ID);
            setNames (organismE, PSIMITags.TAXID_UN_NAME, PSIMITags.TAXID_UN_NAME);
        }

        //add organism now - for semantics
        if (organismE.getChildren ().size() > 0) {
            interactorE.addContent(organismE);
        }

       //check to see the interactor has an xref
       //TODO: Problem, even for interactor of type unidentified psicquic is looking for
       // an identifier.  Need to put in a dummy identifier so interactors don't get excluded.
       if(interactorE.getChild(PSIMITags.XREF, this.psins) == null)
            this.badInteractionData = true;
       return interactorSpeciesID;
    }

    /* method to translate the BIND places
    * Given an individual place and corresponding PSI-MI compartment object populate all the relevant information
    *   Currently get the place description, gen place CV (translates it to GO term) and spec place other-db term
    *
    * @param compartment - the PSI MI object to store an individual place, a compartment.
    * @param place - the BIND place
    * @param UniqueParent
    * @param identifier - id associated with the object where the compartment is found (either an interactor or an interaction)
    *                       (not an important parameter.  It is just needed to identify the external GO terms that is annotated in the place
                             if the publication is not assocaited with the interaction.)
    * @param interaction_pmids - a list of the current pubmeds associated with the interaction (extracted from the BIND-source)
    *
    * @return updated compartment object
    */
    protected Element translatePlace (Element compartment, Element  placeE, String UniqueParent,
                                      String ID, ArrayList<String> interaction_pmids) throws JDOMException, IOException {

        //get the description associated with this place
        Element descr = placeE.getChild(BINDTags.PLACE_DESCR);

        //although the general place is not an optional term for a place, if there is a specific place
        //it will be a better primary reference than the general place.

       /*There are two forms the outher-db can be formatted:
       1.  <BIND-other-db>
             <BIND-other-db_dbname>Gene Ontology</BIND-other-db_dbname>
             <BIND-other-db_intp>0</BIND-other-db_intp>
             <BIND-other-db_strp>GO:0016023 [IDA]</BIND-other-db_strp>
           </BIND-other-db>
       2. <BIND-other-db>
            <BIND-other-db_dbname>Gene Ontology</BIND-other-db_dbname>
            <BIND-other-db_intp>9276</BIND-other-db_intp>
            <BIND-other-db_strp>IDA</BIND-other-db_strp>
          </BIND-other-db>
       */
        String otherDBName = null;
        String otherDBID = null;      //either STRP or INTP
        String otherDBDesc = null;

        ArrayList<String> uniqueParents = new ArrayList<String>();
        uniqueParents.add(placeE.getName());
        uniqueParents.add(UniqueParent);

        ArrayList <Element> otherDBList = XMLUtil.getElement (BINDTags.BIND_SPEC_PLACE_OTHERDB, placeE,uniqueParents,
                                   XMLUtil.getDepth(placeE.getName(),uniqueParents));

        if (XMLUtil.validateExists(BINDTags.OTHERDB, otherDBList)) {
            for (int i = 0; i < otherDBList.size (); i++) {
                Element otherDB = otherDBList.get(i).getChild(BINDTags.OTHERDB);
                otherDBName = otherDB.getChildText(BINDTags.OTHERDB_NAME);
                otherDBDesc = otherDB.getChildText(BINDTags.OTHERDB_DESC);
                //make sure the DBName is Gene Ontology
                //had to add the two other names for GO because of lack of consistancy in BIND specification of GO.
                if(otherDBName.equalsIgnoreCase(PSIMITags.REFTYPE_GO) || otherDBName.equalsIgnoreCase("GO") ||  otherDBName.equalsIgnoreCase("GeneOntology")){

                  otherDBName = PSIMITags.REFTYPE_GO;
                  if (otherDB.getChild (BINDTags.OTHERDB_STRP) != null) {
                        otherDBID = getDefText (otherDB.getChild(BINDTags.OTHERDB_STRP), NO_VAL);

                        if (otherDBID.equals (NO_VAL))
                            otherDBID = null;

                        //check to see if the code is embedded in the id.  If it is then get rid of it
                        else
                            otherDBID = otherDBID.split("\\[")[0];
                    }
                    //if the intp value is zero
                    if ((otherDB.getChild(BINDTags.OTHERDB_INTP) != null) || (!otherDB.getChildText(BINDTags.OTHERDB_INTP).equals("0"))) {
                        if (XMLUtil.validateInt (otherDB.getChild (BINDTags.OTHERDB_INTP))) {

                                //get the integer value
                                otherDBID = otherDB.getChildText(BINDTags.OTHERDB_INTP);

                                for (int j = otherDBID.length (); j < BINDTags.GO_PAD_LEN; j++) {
                                    otherDBID = BINDTags.GO_PAD_STR + otherDBID;
                                }
                                otherDBID = BINDTags.GO_PREFIX + otherDBID;
                        }
                    }
                }//other db name is GO
                else
                   System.err.println ("There is a special place other-db references a database other than GO.  It references " + otherDBName);
            } //for loop
        }

        //for each place defined there has to be a general place defined, and there has to be a start location
        ArrayList <Element> gen_place = XMLUtil.getElement (BINDTags.GEN_PLACE,
                placeE,uniqueParents, XMLUtil.getDepth(placeE.getName(),uniqueParents));

         if (XMLUtil.validateExists(BINDTags.GEN_PLACE, gen_place)) {
            //get the type of gen-place by the name of the tag of the child of gen_place
            List current_places = gen_place.get(0).getChildren();
            String place_name;
            Element current_place;

            //A Bind-gen-place consists of a start and an end.  The start is mandatory and the end is optional.
            if(current_places.size()>2)
                System.err.println ("There is more than two locations defined in "+ BINDTags.GEN_PLACE);
            else{

                //currently only parse the start location.
                //TODO: Add support for end place.
                current_place = (Element)current_places.get(0);
                place_name = current_place.getName();
                Element type = null;

                //need to get the specific type for this place.
                //if the child is of type BIND-membrane or BIND-localize we can just get the value of the attribute associated with the term
                if(current_place.getChildren().size() > 0)
                    type = (Element)(current_place.getChildren().get(0));

                //no additional tags under this type
                if(type == null)
                    place_name = place_name;
                //bind membrane and bind localize have the same struct.  They just have a tag and a attribute value.
                else if((type.getName().equals(BINDTags.BIND_MEMBRANE)) || (type.getName().equals(BINDTags.BIND_LOCALIZE)))
                    place_name = place_name + "_" + type.getAttributeValue("value");
                else if(type.getName().equals(BINDTags.BIND_CHLOR) ){

                }
                else if(type.getName().equals(BINDTags.BIND_DMO) ){
                    //get the child term
                    Element dmo_type = (Element)type.getChildren().get(0);
                    place_name = place_name + "_" + dmo_type.getName();

                    Element dmo_type_membrane = dmo_type.getChild(BINDTags.BIND_MEMBRANE);

                    if( dmo_type_membrane != null){
                        place_name = place_name + "_" + dmo_type_membrane.getAttributeValue("value");
                    }

                }
                else if(type.getName().equals(BINDTags.BIND_CILFLAG) ){
                    //get the child term
                    Element cilflag_type = (Element)(type.getChildren().get(0));
                    //place_name = cilflag_type.getName();
                    //place_name = place_name + "_" + cilflag_type.getName();

                    Element cilflag_type_memloc = null;

                    //the children can ben either a bind-membrane or bind-localize and both are treated the same way
                    //the type membrane or inside is already specified in cilflag_type name.
                    if(cilflag_type.getChildren().size() > 0)
                        cilflag_type_memloc = (Element)cilflag_type.getChildren().get(0);

                    if( cilflag_type_memloc != null){
                        place_name = place_name + "_" + cilflag_type_memloc.getAttributeValue("value");

                    }

                }
                CVMapEntry CVMapper = CVMapEntry.getMappedEntry(place_name);
                if(CVMapper == null)
                    System.err.println("Can't find CV term for " + place_name);

                //if we have values from the special place then make the description the full name
                if(otherDBDesc != null)
                    setNames (compartment, CVMapper.getPSIMITerm_SL(),otherDBDesc);
                else
                    setNames (compartment, CVMapper.getPSIMITerm_SL(),CVMapper.getPSIMITerm_FL());

                //gen places are mapped to GO terms so the exref is from GO db and the type of reference is
                //"gene onotology cellular compartment"
                if(otherDBID != null){
                    setXRef(compartment, PSIMITags.PREF, PSIMITags.REFTYPE_GO, PSIMITags.REFTYPE_GO_ID,
                        otherDBID, PSIMITags.REFTYPE_CC_ID, PSIMITags.REFTYPE_CC,null);
                    setXRef(compartment, PSIMITags.SREF, PSIMITags.REFTYPE_GO, PSIMITags.REFTYPE_GO_ID,
                        CVMapper.getTargetID(), PSIMITags.REFTYPE_CC_ID, PSIMITags.REFTYPE_CC,null);
                }
                else{
                    setXRef(compartment, PSIMITags.PREF, PSIMITags.REFTYPE_GO, PSIMITags.REFTYPE_GO_ID,
                        CVMapper.getTargetID(), PSIMITags.REFTYPE_CC_ID, PSIMITags.REFTYPE_CC,null);
                }
                if(descr != null)
                    addAttribute(compartment, PSIMITags.ATT_COMPARTMENT_DESC,descr.getText());

            }

             //get the source for this place
            ArrayList <Element> sources_place = XMLUtil.getElement (BINDTags.PLACE_SOURCE,
                placeE,uniqueParents, XMLUtil.getDepth(placeE.getName(),uniqueParents));
            if(XMLUtil.validateExists(BINDTags.PLACE_SOURCE, sources_place)){
                //create a dummy object to store the xrefs
                Element dummyE = new Element ("dummyElement", this.psins);

                //get the pubs
                ArrayList <Element> sources = XMLUtil.getElement (BINDTags.PUB_OBJ,
                placeE,uniqueParents, XMLUtil.getDepth(placeE.getName(),uniqueParents));

                //create a list to store the new pubmeds with this place

                //for each of the pubs, check to see if any of the publcations are in the interaction subsets
                for (int i = 0; i < sources.size (); i++) {
                    ArrayList<String> placePmids = new ArrayList<String>();

                    placePmids = translatePubMed(dummyE,sources.get(i),placePmids, uniqueParents, false);

                    //get the description on the publication
                    ArrayList <Element> source_descr = XMLUtil.getElement (BINDTags.PUB_DESCR,
                        sources.get(i),uniqueParents, XMLUtil.getDepth((sources.get(i)).getName(),uniqueParents));
                    String pub_descr = "";
                    if(XMLUtil.validateExists(BINDTags.PUB_DESCR, source_descr)){
                        pub_descr = source_descr.get(0).getText();
                    }
                    //check to see if the placePmids are unique to the place.
                    for(int k=0; k<placePmids.size();k++){
                        //if the pubmed is not in the list of current publications, then it is unique to the places.
                        if(!interaction_pmids.contains(placePmids.get(k))){
                            Novel_GO_terms.write(ID + "\t" + otherDBName +"\t"+ otherDBID + "\t" + otherDBDesc + "\t" + placePmids.get(k) +"\t" + pub_descr + "\n");
                            Novel_GO_terms.flush();
                        }
                    }
                }

            }

            return compartment;
        }
         else
             System.err.println("There is a place defined but no gen-place defined.");



        return null;

    }

    /*
    * Method to translate BIND-descr
    *
    * @param - interactionE - interaction element for the psi-mi conversion
    * @param - BINDdescr - BIND description to parse
    * @param - BINDInteraction - BIND interaction to parse
    *
    * @return - the populated psi-mi interaction Element.
    */
    private Element translateDescr(Element interactionE, Element BINDdescrE, Element BINDINteraction) throws JDOMException{

        //The place associated with BIND-Descr is related to the place of the interaction
        //In psi-mi there is no compartment associated with interaction.
        // It needs to be put in the experiment description -> hostOrganism


        //get intramolecular status
        //only if intramolecular is true create a tag in the interaction
        ArrayList <Element> intramolecularList = XMLUtil.getElement (BINDTags.DESC_INTRAMOL, BINDdescrE,
                                   XMLUtil.getDepth(BINDdescrE.getName()));
        if(XMLUtil.validateSingle(BINDTags.DESC, intramolecularList)){
            String flag = intramolecularList.get(0).getAttributeValue("value");
            if(flag.equalsIgnoreCase("true")){
                //create new element
                Element intramolecular = new Element (PSIMITags.INTRA, this.psins);
                intramolecular.addContent(flag);

                //add element to psi-mi interaction
                interactionE.addContent(intramolecular);
            }
        }

        return null;
    }


    /*
    *  Method to translate the BIND interator type
    *  BIND interactor types are protein, rna, dna, gene, complex, photon, small molecular, not specified.
    *
    * @param interactorTypeE - interactorType element for the psi-mi conversion
    * @param BINDInteractorE - BIND interactor.
    */
    private Element translateInteractorType (Element interactorTypeE, Element
                                    BINDInteractorE) throws JDOMException {

        //protein
        ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_P,
                BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_P, tempList)) {
            CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_P);
            setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                       CVMapper.getPSIMITerm_FL());
            setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                    CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
            //temp section to limit trans support for proteins
            if (BINDInteractorE.getName().equals(BINDTags.INTERACT_A)) {
                this.proteinAFlag = true;
            } else if (BINDInteractorE.getName().equals(BINDTags.INTERACT_B)) {
                this.proteinBFlag = true;
            }
            return tempList.get (0);
        }
        //Gene
        //skip for complexes, no such type for complexes in BIND
        if (!BINDInteractorE.getName().equals (BINDTags.COMPLEX_MOL_OBJ)) {
            tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_GENE,
                       BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_GENE, tempList)) {
                CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_GENE);
                setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                           CVMapper.getPSIMITerm_FL());
                setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                        CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                return tempList.get (0);
            }
        }
        //DNA
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_DNA,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_DNA, tempList)) {
            CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_DNA);
            setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                       CVMapper.getPSIMITerm_FL());
            setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                    CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
            return tempList.get (0);
        }
        //RNA
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_RNA,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_RNA, tempList)) {
            CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_RNA);
            setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                       CVMapper.getPSIMITerm_FL());
            setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                    CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
            return tempList.get (0);
        }
        //Complex
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_COM,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_COM, tempList)) {
            CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_COM);
            setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                       CVMapper.getPSIMITerm_FL());
            setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                    CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
            return tempList.get (0);
        }
        //Small Molecule
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_SM,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_SM, tempList)) {
            CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_SM);
            setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                       CVMapper.getPSIMITerm_FL());
            setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                    CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
            return tempList.get (0);
        }
        //not specified
        //skip for complexes, no such type for complexes in BIND
        if (!BINDInteractorE.getName().equals (BINDTags.COMPLEX_MOL_OBJ)) {
            tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_NS,
                       BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_NS, tempList)) {
                CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_NS);
                setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                           CVMapper.getPSIMITerm_FL());
                setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                        CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                return tempList.get (0);
            }
        }
        //Photon - no match in PSI-MI
        //skip for complexes, no such type for complexes in BIND
        if (!BINDInteractorE.getName().equals (BINDTags.COMPLEX_MOL_OBJ)) {
            tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_PH,
                       BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_PH, tempList)) {
                CVMapEntry CVMapper = CVMapEntry.getMappedEntry(BINDTags.OBJ_TYPE_ID_PH);
                setNames (interactorTypeE, CVMapper.getPSIMITerm_SL(),
                                           CVMapper.getPSIMITerm_FL());
                setXRef(interactorTypeE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                        CVMapper.getTargetID(), PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
                //System.err.println ("No support for 'photon' type in PSI-MI.");
                return tempList.get (0);
            }
        }
        System.err.println ("Unsupported interactant type for: " +
                            BINDInteractorE.getName());
        return null;
    }

    /*
    *  Method to getthe BIND interator type  - helps coolect stats about the interactions
    *  BIND interactor types are protein, rna, dna, gene, complex, photon, small molecular, not specified.
    *
    * @param interactorTypeE - interactorType element for the psi-mi conversion
    * @param BINDInteractorE - BIND interactor.
    */
    private String getBINDInteractorType ( Element
                                    BINDInteractorE) throws JDOMException {

        //protein
        ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_P,
                BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_P, tempList))
            return BINDTags.OBJ_TYPE_ID_P;

        //Gene
        //skip for complexes, no such type for complexes in BIND
        if (!BINDInteractorE.getName().equals (BINDTags.COMPLEX_MOL_OBJ)) {
            tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_GENE,
                       BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_GENE, tempList)) {
                return BINDTags.OBJ_TYPE_ID_GENE;
            }
        }
        //DNA
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_DNA,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_DNA, tempList)) {
            return BINDTags.OBJ_TYPE_ID_DNA;
        }
        //RNA
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_RNA,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_RNA, tempList)) {
            return BINDTags.OBJ_TYPE_ID_RNA;
        }
        //Complex
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_COM,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_COM, tempList)) {
            return BINDTags.OBJ_TYPE_ID_COM;
        }
        //Small Molecule
        tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_SM,
                   BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
        if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_SM, tempList)) {
            return BINDTags.OBJ_TYPE_ID_SM;
        }
        //not specified
        //skip for complexes, no such type for complexes in BIND
        if (!BINDInteractorE.getName().equals (BINDTags.COMPLEX_MOL_OBJ)) {
            tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_NS,
                       BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_NS, tempList)) {
               return BINDTags.OBJ_TYPE_ID_NS;
            }
        }
        //Photon - no match in PSI-MI
        //skip for complexes, no such type for complexes in BIND
        if (!BINDInteractorE.getName().equals (BINDTags.COMPLEX_MOL_OBJ)) {
            tempList = XMLUtil.getElement (BINDTags.OBJ_TYPE_ID_PH,
                       BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.OBJ_TYPE_ID_PH, tempList)) {
               return BINDTags.OBJ_TYPE_ID_PH;
            }
        }
        System.err.println ("Unsupported interactant type for: " +
                            BINDInteractorE.getName());
        return null;
    }


    private Integer translateInteractorID (Element interactorE, Element
            BINDInteractorE, Element BINDExtRefE, Element BINDNameE,
                             Element BINDInteractorTypeE, int interactorSpeciesID) throws JDOMException,IOException {

        Integer taxid_gi = interactorSpeciesID;

        if (BINDNameE == null || BINDNameE.getText().trim().length () == 0 ||
            BINDNameE.getText().trim().equalsIgnoreCase(BINDTags.UNKNOWN) ||
            BINDNameE.getText().trim().equalsIgnoreCase(BINDTags.NULL_VAL)) {
            return interactorSpeciesID;                                 //no dummy/default refElementName
        }
        //special complex related issue
        if (BINDInteractorTypeE == null) {
            System.err.println ("Missing interactor type: PSI-MI interactor ID: " +
                   interactorE.getAttributeValue (PSIMITags.ID_ATT) + " " + interactorSpeciesID);
            setXRef(interactorE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                   PSIMITags.TYPE_UP_ID, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);

            return interactorSpeciesID;
        }
        String BINDInteractorType = BINDInteractorTypeE.getName();
        if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_NS) ||
            BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_PH)) {
            //This puts an MI term in XRef position which is incorrect.
            //The molecule doesn't have a xref so this should be empty
            //dummy GI equal to zero.
            CVMapEntry CVMapped = null;
            CVMapped = CVMapEntry.getMappedEntry (PSIMITags.ENTREZ_GI_DB);
            setXRef (interactorE, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                        CVMapped.getPSIMITermID (), "0", null,null,null);
            //setXRef(interactorE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.TYPE_UP_ID);
            return interactorSpeciesID;
        }
        boolean pRefSetFlag = false;
        ArrayList <Element> otherDBList;
        String interactorType = null, refType, refTypeAccn;

        //For protein, dna, rna, and gene types
        if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_P) ||
            BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_DNA) ||
            BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_RNA) ||
            BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_GENE)) {
            if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_P)) {
                interactorType = PSIMITags.TYPE_PROTEIN_SL;
            } else if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_DNA)) {
                interactorType = PSIMITags.TYPE_DNA_SL;
            } else if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_RNA)) {
                interactorType = PSIMITags.TYPE_RNA_SL;
            } else if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_GENE)) {
                interactorType = PSIMITags.TYPE_GENE_SL;
            }
            ArrayList <Element> giList = new ArrayList <Element> (),
                                domainIDList = new ArrayList <Element> ();
            //get the gi
            Element BINDObjectID = BINDInteractorTypeE.getChild (BINDTags.BIND_ID);
            if (BINDObjectID.getChild(BINDTags.BIND_ID_GI) != null) {
                giList.addAll (BINDObjectID.getChild(BINDTags.BIND_ID_GI).
                               getChildren (BINDTags.ID_GI));
            }

            //get the entrez gene id
            if (BINDObjectID.getChild(BINDTags.BIND_ID_DI) != null) {
                domainIDList.addAll (BINDObjectID.getChild(BINDTags.BIND_ID_DI).
                                     getChildren (BINDTags.DOMAIN_ID));
            }


            //TODO: "Rashad comment --> only one, if any" --> why only one?  What happens if there are multiple GIs??  Is that possible?
            if (XMLUtil.validateSingle(BINDTags.ID_GI, giList) &&
                XMLUtil.validateInt(giList.get (0))) {      //only one, if any
                //map GI history from id1fetch if id mapping turned off
                if(!isMapIDs())
                    taxid_gi = mapXref_id1fetch (interactorE, interactorType, ExtResource.LIST_GI,
                         giList.get (0).getTextTrim (), interactorSpeciesID);
                else
                    taxid_gi = mapXref (interactorE, interactorType, ExtResource.LIST_GI,
                         giList.get (0).getTextTrim (), interactorSpeciesID);
                pRefSetFlag = true;

                //Add all the GIs to the list of unique GIs.
                //Keep a count of how many times we see each GI
                if(GI_index != null){
                    //If this method is called from the complex translation
                    for(int r = 0; r < giList.size();r++){
                        //check to see if the gi is already in the list
                        if(GI_index.get(giList.get(r).getTextTrim()) == null)
                            GI_index.put(giList.get (r).getTextTrim (), 1);
                        else{
                            Integer count = GI_index.get(giList.get (r).getTextTrim ());
                            count++;
                            GI_index.put(giList.get (r).getTextTrim (), count);
                        }

                    }
                }
            }

            if (XMLUtil.validateSingle(BINDTags.DOMAIN_ID, domainIDList) &&
                XMLUtil.validateInt(domainIDList.get (0))) {    //only one, if any
                CVMapEntry CVMapped = CVMapEntry.getMappedEntry(PSIMITags.ENTREZ_GENE_DB);
                refType = CVMapped.getAddInfo(interactorType, domainIDList.get (0).getTextTrim());
                refTypeAccn = PSIMITags.getRefTypeAccn (refType);
                if (!pRefSetFlag) {
                    setXRef (interactorE, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                             CVMapped.getPSIMITermID (),
                             domainIDList.get (0).getTextTrim(), refTypeAccn, refType,null);
                    //map GI history from id1fetch if id mapping turned off
                    if(!isMapIDs())
                        mapXref_id1fetch (interactorE, interactorType, ExtResource.LIST_ID,
                             domainIDList.get (0).getTextTrim(), interactorSpeciesID);
                    else
                        mapXref (interactorE, interactorType, ExtResource.LIST_ID,
                             domainIDList.get (0).getTextTrim(), interactorSpeciesID);
                    pRefSetFlag = true;
                } else {
                    setXRef (interactorE, PSIMITags.SREF, CVMapped.getPSIMITerm_SL(),
                             CVMapped.getPSIMITermID (),
                             domainIDList.get (0).getTextTrim(), refTypeAccn, refType,null);
                    //map GI history from id1fetch if id mapping turned off
                    if(!isMapIDs())
                        mapXref_id1fetch (interactorE, interactorType, ExtResource.LIST_ID,
                             domainIDList.get (0).getTextTrim(), interactorSpeciesID);
                     else
                        mapXref (interactorE, interactorType, ExtResource.LIST_ID,
                             domainIDList.get (0).getTextTrim(), interactorSpeciesID);
                }
            }
        } else if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_SM)) {
            interactorType = PSIMITags.TYPE_SM_SL;
            ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.SMALL_MOLECULE_ID,
                  BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.SMALL_MOLECULE_ID, tempList) &&
                XMLUtil.validateInt(tempList.get (0))) {    //only one, if any, internal ID
                CVMapEntry CVMapped = CVMapEntry.getMappedEntry(PSIMITags.BIND_SL);
                //CVMapEntry CVMapped = CVMapEntry.getMappedEntry(PSIMITags.BIND_SMID_SL);
                refType = CVMapped.getAddInfo(interactorType,
                                              tempList.get (0).getTextTrim ());
                refTypeAccn = PSIMITags.getRefTypeAccn(refType);
                setXRef (interactorE, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                         CVMapped.getPSIMITermID (),
                         tempList.get (0).getTextTrim(), refTypeAccn, refType,null);
                pRefSetFlag = true;
            }
        } else if (BINDInteractorType.equals(BINDTags.OBJ_TYPE_ID_COM)) {
            interactorType = PSIMITags.TYPE_COMPLEX_SL;
            ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.COMPLEX_ID,
                  BINDInteractorE, XMLUtil.getDepth(BINDInteractorE.getName()));
            if (XMLUtil.validateSingle(BINDTags.COMPLEX_ID, tempList) &&
                XMLUtil.validateInt(tempList.get (0))) {    //only one, if any
                CVMapEntry CVMapped = CVMapEntry.getMappedEntry(PSIMITags.BIND_COMPLEXID_SL);
                refType = CVMapped.getAddInfo(interactorType,
                                              tempList.get (0).getTextTrim ());
                refTypeAccn = PSIMITags.getRefTypeAccn(refType);
                setXRef (interactorE, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                         CVMapped.getPSIMITermID (),
                         tempList.get (0).getTextTrim(), refTypeAccn, refType,null);
                pRefSetFlag = true;
            }
        }

        if (BINDExtRefE != null) {
            //otherDBList = XMLUtil.getElement (BINDTags.OTHERDB, BINDInteractorE,
            //                       XMLUtil.getDepth(BINDInteractorE.getName()));
            //otherDBList = XMLUtil.getElement (BINDTags.OTHERDB, BINDExtRefE,XMLUtil.getDepth(BINDExtRefE.getName()),true);

            //create a list of the unique parents
            ArrayList<String> uniqueParents = new ArrayList<String>();
            uniqueParents.add(BINDInteractorE.getName());
            uniqueParents.add(BINDExtRefE.getName());

            otherDBList = XMLUtil.getElement (BINDTags.OTHERDB, BINDExtRefE,uniqueParents, XMLUtil.getDepth(BINDExtRefE.getName(), uniqueParents));
            if (XMLUtil.validateExists(BINDTags.OTHERDB, otherDBList)) {

                for (int i = 0; i < otherDBList.size (); i++) {

                    //check the parent, if this an external reference associated with spec place then skip it
                    if(otherDBList.get(i).getParentElement().getName().equalsIgnoreCase(BINDTags.SPEC_PLACE_OTHERDB))
                        continue;

                    String otherDBName = otherDBList.get (i).getChildText(BINDTags.OTHERDB_NAME);
                    String otherDBID = null;      //either STRP or INTP
                    if (otherDBList.get (i).getChild (BINDTags.OTHERDB_STRP) != null) {
                        otherDBID = getDefText (
                            otherDBList.get (i).getChild(BINDTags.OTHERDB_STRP), NO_VAL);
                        if (otherDBID.equals (NO_VAL)) {
                            otherDBID = null;
                        }
                    }
                    if (otherDBList.get (i).getChild(BINDTags.OTHERDB_INTP) != null) {
                        if (XMLUtil.validateInt (otherDBList.get (i).getChild (BINDTags.OTHERDB_INTP))) {
                            if (otherDBID != null) {            //already set
                                otherDBID += "(" +
                                             otherDBList.get (i).getChildText(BINDTags.OTHERDB_INTP)
                                             + ")";
                                System.err.println ("Bad xref format: " + otherDBName + " " + otherDBID);
                            } else {
                                otherDBID = otherDBList.get (i).getChildText(BINDTags.OTHERDB_INTP);
                            }
                        }
                    }
                    if (otherDBID != null) {              //because of bad input
                        CVMapEntry CVMapped = CVMapEntry.getMappedEntry(otherDBName);
                        refType = CVMapped.getAddInfo(interactorType, otherDBID);
                        refTypeAccn = PSIMITags.getRefTypeAccn (refType);
                        //System.out.println (CVMapped.toString());
                        if (!pRefSetFlag) {
                            setXRef (interactorE, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                                     CVMapped.getPSIMITermID (), otherDBID,
                                     refTypeAccn, refType,null);
                            pRefSetFlag = true;
                        } else {
                            setXRef (interactorE, PSIMITags.SREF, CVMapped.getPSIMITerm_SL(),
                                     CVMapped.getPSIMITermID (), otherDBID,
                                     refTypeAccn, refType,null);
                        }
                    }
                }//for loop
            }
        }

        if (!pRefSetFlag) {
            setXRef(interactorE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                               PSIMITags.TYPE_UP_ID, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
            this.badInteractionData = true;
            error_log.write("(PSI-MI ID): " +
                                interactorE.getAttributeValue(PSIMITags.ID_ATT) + "\t No pRef for interactor \n");
            //System.err.println ("No pRef for interactor (PSI-MI ID): " +
                                //interactorE.getAttributeValue(PSIMITags.ID_ATT));
            return taxid_gi;
        }

        return taxid_gi;
    }

    protected void addAttribute (Element parent, String attName, String attValue) {

        if (attValue.length () == 0) {
            System.err.println ("Warning: Dummy attribute value for: " + attName);
            return;
        }
        //System.out.println (attName + GenUtil.COLON + attValue);
        Element attListE = parent.getChild (PSIMITags.ATTLIST, this.psins);
        if (attListE == null) {                            //is first att?
            attListE = new Element (PSIMITags.ATTLIST, this.psins);
            parent.addContent (attListE);
        }
        Element attE = new Element (PSIMITags.ATT, this.psins);
        attE.setAttribute(PSIMITags.NAME_ATT, attName);
        attE.setText(attValue);
        attListE.addContent(attE);
    }

    /*
    * mapXref - creates a external references with the given information
    * <xref><primaryRef db="genbank indentifier" dbAc="MI:0860" id="46015486" refTypeAc="MI:0361" refType="see-also"></primaryRef></xref>
    *
    * parent - parent element to put the xref into
    * interactorType - type (protein, rna, dna, small moeclule...)
    * sourceType - the database the interactor comes from (i.e. uniprot), psimi term associated with source
    * sourceID - psimi identifier for the source
    * interactorSpeciesID - species ID.
    *
    */

    protected Integer mapXref (Element parent,String interactorType, String sourceType,
                            String sourceID, int interactorSpeciesID) throws IOException{

        CVMapEntry CVMapped = null;
        String refType, refTypeAccn;
        //make them all generic GIs, since no mapping info prepared
        if (!isMapIDs()) {
           if (sourceType.equals (ExtResource.LIST_GI)) {
               CVMapped = CVMapEntry.getMappedEntry (PSIMITags.ENTREZ_GI_DB);
               refType = CVMapped.getAddInfo(interactorType, sourceID);
               refTypeAccn = PSIMITags.getRefTypeAccn (refType);
               setXRef (parent, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                        CVMapped.getPSIMITermID (), sourceID, refTypeAccn, refType,null);
           }
           return interactorSpeciesID;
        }
        ArrayList <String> uniprotList,
                  supUniprotList = new ArrayList <String> (),
                  refSeqProList = new ArrayList <String> (),
                  refSeqNucList = new ArrayList <String> (),
                  genProList = new ArrayList <String> ();
        //applies to all source Types
        if (interactorSpeciesID != Integer.parseInt (PSIMITags.TAXID_UN_ID)) {
            uniprotList = IdentifierMapperService.getUniprotIDByXref(sourceType, sourceID,
                        interactorSpeciesID);
        } else {
            uniprotList = IdentifierMapperService.getUniprotIDByXref (sourceType, sourceID);
        }
        //get the GI x-refs based on available mappings
        String sourceTerm = null;
        if (sourceType.equals (ExtResource.LIST_GI)) {
            //get the protein refseqs from the GI
           refSeqProList = IdentifierMapperService.getXrefLO (sourceType, sourceID,
                           ExtResource.LIST_REFSEQ_PRO);
            //get the nucleotide refseqs from the GIs
            refSeqNucList = IdentifierMapperService.getXrefLO (sourceType, sourceID,
                           ExtResource.LIST_REFSEQ_RNA);
            //Get the updated GIs from Batch Entrez
            genProList = IdentifierMapperService.getXrefLO(sourceType, sourceID,
                           IdentifierMapperService.LO_PROTEIN);
            //Get the update refseq nucleotide accession from Batch Entrez
           supUniprotList = IdentifierMapperService.getXrefLO(sourceType, sourceID,
                            IdentifierMapperService.LO_GI_UNI);
           if (uniprotList.size () > 0 || refSeqProList.size() > 0 ||
               genProList.size() > 0) {
               sourceTerm = PSIMITags.ENTREZ_GI__PRO_DB;
           } else if (refSeqNucList.size () > 0) {
               sourceTerm = PSIMITags.ENTREZ_GI_NUC_DB;
           } else {
               sourceTerm = PSIMITags.ENTREZ_GI_DB;
           }
           CVMapped = CVMapEntry.getMappedEntry(sourceTerm);
           refType = CVMapped.getAddInfo(interactorType, sourceID);
           refTypeAccn = PSIMITags.getRefTypeAccn (refType);
           setXRef (parent, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                    CVMapped.getPSIMITermID (), sourceID, refTypeAccn, refType,null);
        }
        //works for any source type
        if (uniprotList.size() > 0) {
            addMappedRefList (parent, interactorType, sourceID,
                           PSIMITags.BIND_UNIPROT_SL, uniprotList);
        }
        if (supUniprotList.size () > 0) {
            addMappedRefList (parent, interactorType, sourceID,
                              PSIMITags.BIND_UNIPROT_SL, supUniprotList);
        }
        if (refSeqProList.size () > 0) {
            addMappedRefList (parent, interactorType, sourceID,
                              PSIMITags.BIND_REFSEQ_P_SL, refSeqProList);
        }
        if (refSeqNucList.size() > 0) {
            addMappedRefList (parent, interactorType, sourceID,
                              PSIMITags.BIND_REFSEQ_M_SL, refSeqNucList);
        }
        if (genProList.size() > 0) {
            addMappedRefList (parent, interactorType, sourceID,
                              PSIMITags.BIND_PRO_SL, genProList);
        }

        return interactorSpeciesID;
    }

/*
    * Given:
    * parent - parent element to put the xref into
    * interactorType - type (protein, rna, dna, small moeclule...)
    * sourceType - the database the interactor comes from (i.e. uniprot), psimi term associated with source
    * sourceID - psimi identifier for the source
    * interactorSpeciesID - species ID.
    *
    * method maps the interactor id use pre-computed id1fetch information
    * annotates each GI with its NCBI status.  If there is an updated GI it addes it as a secondary reference
    * Accession extracted from id1fetch is included as the primary reference with a attribute specifying which db it is from (ie. sp=swissprot, pdb...).
    *
    */
    private Integer mapXref_id1fetch(Element parent,String interactorType, String sourceType,
                                  String sourceID, Integer interactorSpeciesID) throws IOException {

        CVMapEntry CVMapped = null;
        String refType, refTypeAccn;

        //If this id is a GI then get its history
        if (sourceType.equals (ExtResource.LIST_GI)) {
            //get the GI history
            GI_information info = GI_history.get(Integer.parseInt(sourceID));

            //all GIs should have history -- issue error if it is missing.
            if(info == null){
                System.out.println("No history available for GI:" + sourceID);

                //just use the GI
                CVMapped = CVMapEntry.getMappedEntry(PSIMITags.ENTREZ_GI_DB);
                refType = CVMapped.getAddInfo(interactorType, sourceID);
                refTypeAccn = PSIMITags.getRefTypeAccn (refType);
                setXRef (parent, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                    CVMapped.getPSIMITermID (), sourceID, refTypeAccn, refType, PSIMITags.ID_CONVERSION_TYPE, PSIMITags.ORIGINAL_GI,"");

            }

            else{
                //get the information
                Integer original_gi = info.getGI();
                Integer updated_gi = info.getUpdated_gi();
                Integer taxid = info.getTaxid();
                String accession_version = info.getAccession();
                String version = "";
                String accession  = "";
                if(accession_version != null){
                    String[] accver_tok = accession_version.split("\\.");
                    if(accver_tok.length == 2){
                        accession = accver_tok[0];
                        version = accver_tok[1];
                    }
                    else
                        accession = accession_version;
                }
                else
                    accession = accession_version;




                String state = info.getState();

                String db = info.getDb();
                //System.out.println(sourceID + " "+ interactorType + " " + db);

                //make the accession the primary reference (if there is an accession)
                if((accession != null) && (db != null)){
                    CVMapped = CVMapEntry.getMappedEntry(db);
                    //make sure the db is mapped if it isn't then print Erro
                    if(CVMapped == null){
                        error_log.write("Error 18 - DB not found  " + db + "for sourceID " + sourceID + "\n");
                        CVMapped = CVMapEntry.getMappedEntry(PSIMITags.ENTREZ_GI_DB);
                        refType = CVMapped.getAddInfo(interactorType, sourceID);
                        refTypeAccn = PSIMITags.getRefTypeAccn (refType);
                        setXRef (parent, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                                                CVMapped.getPSIMITermID (), original_gi.toString(), refTypeAccn, refType, PSIMITags.ID_CONVERSION_TYPE_STATUS, state,null);

                    }
                    else{
                         refType = CVMapped.getAddInfo(interactorType, sourceID);
                            //if this is not a supported type for this db -- issue a warning but convert it anyways.
                        if(refType.equalsIgnoreCase("N/A")){
                            error_log.write("BIND record interactor type " + interactorType + " does not match the database specified " + db + "for sourceID " + sourceID + "\n");
                            setXRef (parent, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                                    CVMapped.getPSIMITermID (), accession, null, null,version);
                        }else{
                            refTypeAccn = PSIMITags.getRefTypeAccn (refType);
                            setXRef (parent, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                                CVMapped.getPSIMITermID (), accession, refTypeAccn, refType,version);
                            }
                        //add the original GI (attribute contains the status of the GI)
                            CVMapped = CVMapEntry.getMappedEntry(PSIMITags.ENTREZ_GI_DB);
                        refType = CVMapped.getAddInfo(interactorType, sourceID);
                        refTypeAccn = PSIMITags.getRefTypeAccn (refType);
                        setXRef (parent, PSIMITags.SREF, CVMapped.getPSIMITerm_SL(),
                                CVMapped.getPSIMITermID (), original_gi.toString(), refTypeAccn, refType, PSIMITags.ID_CONVERSION_TYPE_STATUS, state,null);
                    }
                }
                //else if (original_gi != updated_gi){
                    //try the same thing with the updated_gi
                //    mapXref_id1fetch(parent,interactorType,sourceType,updated_gi.toString(),interactorSpeciesID);
                //}
                else{ //make the original GI the primary ref
                    CVMapped = CVMapEntry.getMappedEntry(PSIMITags.ENTREZ_GI_DB);
                    refType = CVMapped.getAddInfo(interactorType, sourceID);
                    refTypeAccn = PSIMITags.getRefTypeAccn (refType);
                    setXRef (parent, PSIMITags.PREF, CVMapped.getPSIMITerm_SL(),
                        CVMapped.getPSIMITermID (), original_gi.toString(), refTypeAccn, refType, PSIMITags.ID_CONVERSION_TYPE_STATUS, state,null);

                }

                CVMapped = CVMapEntry.getMappedEntry(PSIMITags.ENTREZ_GI_DB);
                refType = CVMapped.getAddInfo(interactorType, sourceID);
                refTypeAccn = PSIMITags.getRefTypeAccn (refType);

                //add the updated GI - only if the updated GI is different from the original GI
                if(original_gi != updated_gi)
                    setXRef (parent, PSIMITags.SREF, CVMapped.getPSIMITerm_SL(),
                        CVMapped.getPSIMITermID (), updated_gi.toString(), refTypeAccn, refType, PSIMITags.ID_CONVERSION_TYPE, PSIMITags.UPDATED_GI,null);

                //check to see that the taxonomy is the same for the BIND record and the interactor
                //if it isn't then issue an error
                if(interactorSpeciesID != taxid){
                    if(interactorSpeciesID.toString().equalsIgnoreCase(PSIMITags.TAXID_UN_ID))
                        error_log.write("BIND record taxonomy information does not match id1fetch taxid for interactor id:\t" + sourceID + "\tBIND record specifies taxid:\t" + interactorSpeciesID + "\tand id1fetch specified taxid:\t" + taxid + "\n");
                    return taxid;

                }

            }

        }

        return interactorSpeciesID;
    }



    //helper method
    protected void addMappedRefList (Element parent, String interactorType, String
                     sourceID, String listType, ArrayList <String> mappedList) throws IOException {

        CVMapEntry CVMapped = CVMapEntry.getMappedEntry(listType);
        String refType = CVMapped.getAddInfo(interactorType, sourceID);
        String refTypeAccn = PSIMITags.getRefTypeAccn (refType);
        for (int i = 0; i < mappedList.size (); i++) {
            setXRef (parent, PSIMITags.SREF, CVMapped.getPSIMITerm_SL(),
                    CVMapped.getPSIMITermID (), mappedList.get (i),
                    refTypeAccn, refType, PSIMITags.CONVERSIONTYPE, listType,null);

            //output the id conversion
            this.interaction_conversions_log.write(sourceID + "\t" + mappedList.get(i)+ "\t" + listType + "\n");
        }


    }

    //mostly geared for BIND's bad data.
    protected String getDefText (Element element, String defValue) {

        if (element.getText().trim().length () > 0 &&
            !element.getText().trim().equalsIgnoreCase(BINDTags.NULL_VAL) &&
            !element.getText().trim().equals (BINDTags.NULL_VAL2) &&
            !element.getText().trim ().equals (BINDTags.NULL_VAL3) &&
            !element.getText().trim ().contains (BINDTags.NULL_VAL4) &&
            !element.getText().trim ().equalsIgnoreCase(BINDTags.NULL_VAL5) &&
            !element.getText().trim ().equalsIgnoreCase(BINDTags.NULL_VAL6) &&
            !element.getText().trim ().equals(BINDTags.NULL_VAL7)) {
            return element.getText().trim();
        } else {
           //System.err.println ("Invalid text for element: " +
           //       element.getName() + " " + element.getText().trim());

           return defValue;
        }
    }

    protected void addSpecies (int speciesID) {

        if (speciesID > 0) {
            if (!this.speciesList.contains(String.valueOf(speciesID))) {
                this.speciesList.add(String.valueOf(speciesID));
            }
        }
    }

    protected void listSpecies () throws Exception {

        System.out.println ("Dumping species list: " + speciesList.size());
        for (int i = 0; i < speciesList.size(); i++) {
            String speciesIDStr = speciesList.get (i);
            String speciesStr = DWUtil.dumpSpeciesNames(Integer.parseInt(speciesIDStr));
            System.out.println ("Species: " + GenUtil.TAB +
                                speciesIDStr + GenUtil.TAB + speciesStr);
        }
    }

    private void countInteractionType(String interactorA_type, String interactorB_type){

        String interaction_type = interactorA_type + "_" + interactorB_type;

        //check to see if this type of interaction is in the hash list yet
        if(interactionTypeCount != null)
            if(interactionTypeCount.containsKey(interaction_type)){
                Integer interaction_count = interactionTypeCount.get(interaction_type);
                interaction_count++;
                interactionTypeCount.put(interaction_type, interaction_count);
            }
            else
                interactionTypeCount.put(interaction_type, 1);

    }

    protected void addExperimentalRole(Element partE, String type){
            Element experRoleListE = new Element (PSIMITags.EXPERROLELIST, this.psins);

            Element experRoleE = new Element(PSIMITags.EXPERROLE, this.psins);
            experRoleListE.addContent(experRoleE);

            if(type.equalsIgnoreCase("bait")){
                setNames(experRoleE,PSIMITags.SL_BAIT,PSIMITags.SL_BAIT);
                setXRef(experRoleE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                    PSIMITags.ID_BAIT, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);

            }
            else if(type.equalsIgnoreCase("prey")){
                setNames(experRoleE,PSIMITags.SL_PREY,PSIMITags.SL_PREY);
                setXRef(experRoleE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                                    PSIMITags.ID_PREY, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);

            }
            else{
                setNames(experRoleE,PSIMITags.SL_US,PSIMITags.SL_US);
                setXRef(experRoleE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                      PSIMITags.ID_US, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);
            }

            partE.addContent(experRoleListE);

    }

    public int getCounter() {
        return counter;
    }

    public void setCounter(int counter) {
        this.counter = counter;
    }
}
