package org.pathwaycommons.trans;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.TreeMap;

import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;

import org.pathwaycommons.trans.util.XMLUtil;
import org.pathwaycommons.trans.util.GI_information;
import org.genemania.dw.util.GenUtil;

/**
 * BIND To PSI-MI translation for complexes. Modelled as a subclass to the main
 * BIND translator.
 *
 * @author rashadbadrawi
 */

public class BINDToPSIMITransComplex extends BINDToPSIMITrans {

    private int cntTotalComplex, cntTransComplex, cntSkippedComplex,
                cntBadComplex;
    private String BINDComplexID;

    private int counter;

    private BINDToPSIMITrans current_translator_instance;

    //for easier handling

    private HashMap<String, Element> interaction_index;

    private HashMap<String, Integer> GI_index;

    //private HashMap<Integer, GI_information> GI_history;

    //if set to true all complexes will be flattened and inferred interactions will be used
    private boolean flatten_all = true;

     //base constructor.
     protected BINDToPSIMITransComplex (){
        super();
     }

    //overlooking the use of 'super'
    protected BINDToPSIMITransComplex (String inputFilesPath, String fileName, Namespace psins,
             int count, BufferedWriter bw, BufferedWriter missing_pmids
            , BufferedWriter Novel_GO_terms, BufferedWriter error_log, BufferedWriter stats_log,MissingPMIDsFile new_pmids,
              HashMap<String, Element> interaction_index, BINDToPSIMITrans trans, HashMap<String, Integer> GI_index,
              HashMap<Integer, GI_information> GI_history, TreeMap<String, String> interactionsMap, TreeMap<String, String> badInteractionsMap) {

        super ();
        this.psins = psins;
        this.fileName = fileName;
        this.inputFilesPath = inputFilesPath;
        this.bw = bw;
        this.missing_pmids = missing_pmids;
        this.Novel_GO_terms = Novel_GO_terms;
        this.error_log = error_log;
        this.stats_log = stats_log;
        this.new_pmids = new_pmids;
        this.interaction_index = interaction_index;
        this.GI_index = GI_index;
        this.GI_history = GI_history;

        this.interactionsMap = interactionsMap;
        this.badInteractionsMap = badInteractionsMap;

        this.counter = count;
        this.current_translator_instance = trans;
    }

    protected void translateAllComplexes (Element entryE, Element BINDSubmitE)
                                       throws JDOMException, IOException {

        this.counter = current_translator_instance.getCounter();

        Element interactionListE = entryE.getChild(PSIMITags.INT_LIST, this.psins);
        //if no interactions in this file/species
        if (interactionListE == null) {
            interactionListE = new Element (PSIMITags.INT_LIST, this.psins);
            entryE.addContent (interactionListE);
        }
        ArrayList <Element> bindIntList = XMLUtil.getElement (BINDTags.COMPLEX,
                    BINDSubmitE, 1);
        for (int i = 0; i < bindIntList.size (); i++) {
            translateComplex (interactionListE, bindIntList.get (i));
        }
        System.out.println ("For file: " + getInputFilesPath() + ": ");
        System.out.println ("Total Complexes: " + this.cntTotalComplex +
                            " Translated: " + this.cntTransComplex +
                            " Skipped: " + this.cntSkippedComplex +
                            " Bad: " + this.cntBadComplex);
        stats_log.write (this.currentTaxID + "\t ");
        stats_log.write ("Total Complexes: " + this.cntTotalComplex +
                            " \tTranslated: " + this.cntTransComplex +
                            " \tSkipped: " + this.cntSkippedComplex +
                            " \tBad: " + this.cntBadComplex + "\n");

        current_translator_instance.setCounter(this.counter);
    }

    public void scanComplexes(HashSet<String> interactionIDs, Element BINDSubmitE)throws JDOMException{
      ArrayList <Element> bindIntList = XMLUtil.getElement (BINDTags.COMPLEX,
                    BINDSubmitE, 1);
        for (int i = 0; i < bindIntList.size (); i++) {
            Element BINDCompE = bindIntList.get (i);


        //get interaction IDs from the interactions list
        ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.COMPLEX_INT_LIST, BINDCompE,
                                       XMLUtil.getDepth (BINDCompE.getName()));
        if (!XMLUtil.validateSingle(BINDTags.COMPLEX_INT_LIST, tempList)) {
            System.err.println ("Warning: No interaction list found for complex: " + BINDComplexID);
        } else {
            ArrayList <Element> interactionIDList;
            interactionIDList = XMLUtil.getElement (BINDTags.INTERACTION_ID, tempList.get (0),
                                           XMLUtil.getDepth (tempList.get (0).getName()));
            if (XMLUtil.validateExists(BINDTags.INTERACTION_ID, interactionIDList)) {
                for (int k = 0; k < interactionIDList.size (); k++) {
                    if (getDefText(interactionIDList.get (k), NO_VAL).equals (NO_VAL)) {
                        System.err.println ("Error 3: Bad interaction ID for complex: " + BINDComplexID);
                        continue;
                    }
                    interactionIDs.add(interactionIDList.get (k).getTextTrim());
                }
            }
        }
        }


    }

    private void translateComplex (Element interactionListE, Element BINDCompE)
                                  throws JDOMException, IOException {

        String complexDesc = NO_VAL, complexNum = NO_VAL, orderedFlag = NO_VAL;
        BINDComplexID = NO_VAL;
        ArrayList <String> interactionIDsList = new ArrayList <String> ();

        //get complex ID
        ArrayList <Element> tempList = XMLUtil.getElement (BINDTags.COMPLEX_ID,
             BINDCompE.getChild(BINDTags.COMPLEX_MCID),
             XMLUtil.getDepth (BINDCompE.getChild(BINDTags.COMPLEX_MCID).getName()));
        if (XMLUtil.validateSingle(BINDTags.COMPLEX_ID, tempList)) {
            BINDComplexID = getDefText(tempList.get (0), NO_VAL);
        }
        if (BINDComplexID.equals (NO_VAL)) {
            System.err.println ("Error 1: NO ID for complex.");
            this.badInteractionData = true;
        }

        //get complex description
        tempList = XMLUtil.getElement (BINDTags.COMPLEX_DESC, BINDCompE,
                                       XMLUtil.getDepth (BINDCompE.getName()));
        if (XMLUtil.validateSingle(BINDTags.COMPLEX_DESC, tempList)) {
            complexDesc = getDefText(tempList.get (0), NO_VAL);
        }
        if (complexDesc.equals (NO_VAL)) {
            System.err.println ("Error 2: NO Desc for complex: " + BINDComplexID);
            this.badInteractionData = true;
        }

        //Total num of subunits. There is another 'mol_obj' subnumber that is never used.
        tempList = XMLUtil.getElement (BINDTags.COMPLEX_NUM, BINDCompE,
                                       XMLUtil.getDepth (BINDCompE.getName()));
        if (XMLUtil.validateSingle(BINDTags.COMPLEX_NUM, tempList)) {
            complexNum = getDefText(tempList.get (0), NO_VAL);
        }
        if (complexNum.equals (NO_VAL)) {
            System.err.println ("Warning: NO count for complex: " + BINDComplexID);
        }

        //get interaction IDs from the interactions list
        tempList = XMLUtil.getElement (BINDTags.COMPLEX_INT_LIST, BINDCompE,
                                       XMLUtil.getDepth (BINDCompE.getName()));
        if (!XMLUtil.validateSingle(BINDTags.COMPLEX_INT_LIST, tempList)) {
            System.err.println ("Warning: No interaction list found for complex: " + BINDComplexID);
        } else {
            ArrayList <Element> interactionIDList;
            interactionIDList = XMLUtil.getElement (BINDTags.INTERACTION_ID, tempList.get (0),
                                           XMLUtil.getDepth (tempList.get (0).getName()));
            if (XMLUtil.validateExists(BINDTags.INTERACTION_ID, interactionIDList)) {
                for (int i = 0; i < interactionIDList.size (); i++) {
                    if (getDefText(interactionIDList.get (i), NO_VAL).equals (NO_VAL)) {
                        System.err.println ("Error 3: Bad interaction ID for complex: " + BINDComplexID);
                        continue;
                    }
                    interactionIDsList.add (interactionIDList.get (i).getTextTrim());
                }
            }
        }

        //complex order flag
        tempList = XMLUtil.getElement (BINDTags.COMPLEX_ORDERED_FLAG, BINDCompE,
                                       XMLUtil.getDepth (BINDCompE.getName()));
        if (XMLUtil.validateSingle(BINDTags.COMPLEX_ORDERED_FLAG, tempList)) {
            orderedFlag = tempList.get (0).getAttributeValue(BINDTags.VALUE_ATT);
        }
        
        //build the PSI-MI complex
        Element interactionE = new Element (PSIMITags.INT, this.psins);
        addComplex (interactionE, BINDCompE, BINDComplexID, complexDesc,
                   complexNum, orderedFlag, interactionIDsList);

        //get/add the complex division type
        tempList = XMLUtil.getElement (BINDTags.INTERACTION_COL, BINDCompE,
                            XMLUtil.getDepth (BINDCompE.getName()));
        if (XMLUtil.validateSingle (BINDTags.INTERACTION_COL, tempList)) {
            String BINDDivision = tempList.get (0).getTextTrim ();
            addAttribute (interactionE, PSIMITags.ATT_BIND_DIVISION,
                          BINDDivision);
        }
        interactionsMap.put (BINDComplexID, 
                             interactionE.getAttributeValue(PSIMITags.ID_ATT));
        //filter bad data
        if (!this.badInteractionData) {
            interactionListE.addContent (interactionE);
        } else {
            this.badInteractionData = false;
            addAttribute (interactionE, PSIMITags.ATT_BAD_DATA_FILE, this.fileName);
            XMLUtil.dump(interactionE, this.bw);
            this.bw.newLine();
            this.cntBadComplex++;
            badInteractionsMap.put (BINDComplexID,
                                    interactionE.getAttributeValue(PSIMITags.ID_ATT));
        }
        this.cntTotalComplex++;
        this.cntTransComplex++;
    }

    //interaction counter increments regardless.
    private void addComplex (Element interactionE, Element BINDCompE,
         String BINDComplexID, String shortLabel, String subunitNum,
         String orderedFlag, ArrayList <String> interactionIDsList) throws JDOMException,IOException {


        interactionE.setAttribute(PSIMITags.ID_ATT,
                              Integer.toString (++this.counter));


        if (!shortLabel.equals (NO_VAL)) {
            setNames (interactionE, shortLabel, shortLabel);
        } else {
            setNames (interactionE, shortLabel, null);
        }
        setCVRef (interactionE, PSIMITags.BIND_SL, PSIMITags.REFTYPE_IDENT,
                  PSIMITags.PREF, BINDComplexID);

        Element expListE = new Element (PSIMITags.EXP_LIST, this.psins);
        interactionE.addContent (expListE);
        //translateExpDesc (expListE, BINDCompE, BINDComplexID);
        Element partListE = new Element (PSIMITags.PARTLIST, this.psins);
        interactionE.addContent (partListE);

        Element inferredListE = new Element(PSIMITags.INFERREDLIST, this.psins);

        //from the list of interactions get the experimental descriptions
        //make sure they are unique.
        boolean flatten = true;
        boolean first_interaction = true;
        boolean inferred_added = false;
        int null_interaction_count = 0;
        for(int k = 0 ; k<interactionIDsList.size();k++){
            //get the current interaction
            Element current_interaction = interaction_index.get(interactionIDsList.get(k));

            if(current_interaction != null){

                if(flatten_all){
                    if(!inferred_added){
                        interactionE.addContent(inferredListE);
                        inferred_added = true;
                    }
                    //translate the experiments from the current interaction (make sure they aren't already entered)
                    translateInteractors(partListE, expListE,inferredListE, current_interaction, k,BINDComplexID);
                }

                else{
                    if(first_interaction){
                        //add the experimental role from Bind-condition_bait-condition
                        //can have one of three values: a-is-bait, b-is-bait,not-applicable
                        ArrayList<Element> eList = XMLUtil.getElement(BINDTags.BAIT_COND, current_interaction, XMLUtil.getDepth(BINDTags.INTERACTION));
                        String bait_condition = "";
                        if (XMLUtil.validateSingle (BINDTags.BAIT_COND, eList)) {
                            bait_condition = eList.get(0).getAttributeValue("value");
                        }
                        //for the first interaction find out if the bait is specified.  If not print out the stats
                        if(bait_condition.equalsIgnoreCase("a-is-bait") || bait_condition.equalsIgnoreCase("b-is-bait")){
                                error_log.write(BINDComplexID + "\tComplex has a bait defined and is a pull down\n");
                            flatten = true;
                        }else if(bait_condition.equalsIgnoreCase("not-applicable") || bait_condition.equalsIgnoreCase("")){
                                error_log.write(BINDComplexID + "\tComplex is not a pull down\n");
                            flatten = false;
                        }
                        else{
                            System.out.println("Bait is not defined correctly.  Value is :" + bait_condition);
                        }

                        first_interaction = false;

                    }

                    if(flatten){
                        if(!inferred_added){
                            interactionE.addContent(inferredListE);
                            inferred_added = true;
                        }
                        //translate the experiments from the current interaction (make sure they aren't already entered)
                        translateInteractors(partListE, expListE, inferredListE,current_interaction, k,BINDComplexID);
                    }
                    else{
                        translateExpDesc (expListE, BINDCompE, BINDComplexID);
                        //get details of each subunit
                        translateMolObj (partListE, BINDCompE, BINDComplexID, interactionIDsList);
                        k = interactionIDsList.size();
                    }

                }
            }
            //the interaction is null --> why? where is the interaction
            else
                null_interaction_count++;
        }


        if(null_interaction_count == interactionIDsList.size())
            badInteractionData= true;

        String intIDStr = "";
        String BINDIntIDStr = "";
        for (int i = 0; i < interactionIDsList.size (); i++) {
            BINDIntIDStr += interactionIDsList.get (i);
            intIDStr += matchIntID (interactionIDsList.get (i));
            if (i < interactionIDsList.size () - 1) {
                BINDIntIDStr += GenUtil.SEMICOLON;
                intIDStr += GenUtil.SEMICOLON;
            }
        }
        addAttribute (interactionE, PSIMITags.ATT_COMPLEX_INT_LIST, intIDStr);
        addAttribute (interactionE, PSIMITags.ATT_COMPLEX_INT_LIST_BIND, BINDIntIDStr);
        addAttribute (interactionE, PSIMITags.ATT_COMPLEX_INT_ORDERED, orderedFlag);
        addAttribute (interactionE, PSIMITags.ATT_COMPLEX_SUBUNIT_CNT, subunitNum);
    }

    //scaled down version for the no-exp cond. BIND interaction.
    private void translateExpDesc (Element expListE, Element BINDCompE, String
                                   BINDComplexID) throws JDOMException,IOException {

        ArrayList<String> addedPubmedids = new ArrayList<String>();

        Element expDescE = new Element (PSIMITags.EXP_DESC, this.psins);
        expDescE.setAttribute(PSIMITags.ID_ATT, Integer.toString(++this.counter));
        expListE.addContent (expDescE);

        //create interaction detection method
        Element intDetMethodE = new Element (PSIMITags.INT_DET_METHOD, this.psins);
        setNames(intDetMethodE, PSIMITags.SL_METHOD_NS, PSIMITags.SL_METHOD_NS);
        setXRef(intDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                PSIMITags.ID_METHOD_NS,PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);

        //create participant detection method
         Element partiDetMethodE = new Element (PSIMITags.PARTI_DET_METHOD, this.psins);
        //inferred by curator method
        setNames(partiDetMethodE, PSIMITags.SL_METHOD_IC, PSIMITags.SL_METHOD_NS);
        setXRef(partiDetMethodE, PSIMITags.PREF, PSIMITags.DB_PSIMI, PSIMITags.ID_PSIMI,
                   PSIMITags.ID_METHOD_IC, PSIMITags.REFTYPE_IDENT_ID, PSIMITags.REFTYPE_IDENT,null);


        //get the pubmed IDs, if any
        translatePubMed(expDescE, BINDCompE.getChild (BINDTags.COMPLEX_REF),addedPubmedids);

        translatePubMed (expDescE, BINDCompE,addedPubmedids);         //order is signifcant

        //if there is no publication associated with this record then put the bind id
        //it will also at least have a general publication
        if (expDescE.getChildren(PSIMITags.BIB, this.psins).size() == 0) {

            //bib ref can be either a pubmed or an attribute list
            //associate the bind id and the title of the general citation
            //in an attribute list

            Element bibRefE = new Element (PSIMITags.BIB, this.psins);

            //requires an xRef for a bibref. Will crash if missing.
            //add dummy pubmid reference

            //don't need the dummy pubmed anymore.
            //CVMapEntry CVMapper = CVMapEntry.getMappedEntry (PSIMITags.PUBMED_SL);
            //setXRef (bibRefE, PSIMITags.PREF, CVMapper.getPSIMITerm_SL(),
            //CVMapper.getPSIMITermID (),
            //PSIMITags.NO_PUBMED,
            //PSIMITags.getRefTypeAccn(PSIMITags.REFTYPE_REF),
            //PSIMITags.REFTYPE_REF);

            //check to see if we can get additional info from general publication record
             ArrayList <Element> PubGenList = XMLUtil.getElement (BINDTags.PUB_GEN,
                            BINDCompE, XMLUtil.getDepth (BINDCompE.getName()));
             if (XMLUtil.validateSingle(BINDTags.PUB_GEN,PubGenList)){
                for(int k=0; k<PubGenList.size();k++){
                    //get the general citation
                    Element gen_cit = PubGenList.get(k).getChild(BINDTags.CIT_GEN);
                    if(gen_cit != null){
                        String title = gen_cit.getChildText(BINDTags.CIT_GEN_TITLE);
                        String cit = gen_cit.getChildText(BINDTags.CIT_GEN_CIT);

                        //check the title to see if we have mapped the title to a pmid
                        if(new_pmids.getPMIDMap().get(title) != null){
                            CVMapEntry CVMapper = CVMapEntry.getMappedEntry (PSIMITags.PUBMED_SL);

                            setXRef (bibRefE, PSIMITags.PREF, CVMapper.getPSIMITerm_SL(),
                                  CVMapper.getPSIMITermID (),
                                  new_pmids.getPMIDMap().get(title),
                                  PSIMITags.getRefTypeAccn(PSIMITags.REFTYPE_REF),
                                  PSIMITags.REFTYPE_REF,null);
                            }
                            else{
                               //write out the publcation that is missing
                               missing_pmids.write("BINDInteractionID:" + BINDComplexID + "\t" + title + "\t" + cit + "\n");
                               addAttribute(bibRefE,PSIMITags.ATT_BIND_Record,BINDComplexID);
                               addAttribute(bibRefE,PSIMITags.ATT_GEN_PUB_TITLE,title);
                            }
                        }
                    }
                }
                else
                    addAttribute(bibRefE,PSIMITags.ATT_BIND_Record,BINDComplexID);
                expDescE.addContent(bibRefE);

          }


        expDescE.addContent (intDetMethodE);
        expDescE.addContent (partiDetMethodE);
    }

    private void translateMolObj (Element partListE, Element BINDCompE, String
          BINDComplexID, ArrayList <String> interactionIDsList) throws JDOMException, IOException {

        ArrayList <Element> bindMolObjList = XMLUtil.getElement (BINDTags.COMPLEX_MOL_OBJ,
                            BINDCompE, XMLUtil.getDepth (BINDCompE.getName()));
        String subUnitNum;
        String BINDIntID, newIntID;
        String interactantAtt;
        Element interactionRefE;
        ArrayList <Element> tempList;
        for (int i = 0; i < bindMolObjList.size (); i++) {
            subUnitNum = NO_VAL;
            interactionRefE = null;
            BINDIntID = null;
            newIntID = null;
            interactantAtt = null;
            //get the subunit number
            tempList = XMLUtil.getElement (
                           BINDTags.COMPLEX_MOLOBJ_ID, bindMolObjList.get (i),
                           XMLUtil.getDepth (bindMolObjList.get (i).getName()));
            if (XMLUtil.validateSingle(BINDTags.COMPLEX_MOLOBJ_ID, tempList)) {
                //zero is a valid complex subunit num
                subUnitNum = tempList.get (0).getTextTrim();
            }
            if (subUnitNum.equals (NO_VAL)) {
                System.err.println ("Error 4: NO ID for subunit for complex: " + BINDComplexID);
            }
            Element partE = new Element (PSIMITags.PART, this.psins);
            partE.setAttribute(PSIMITags.ID_ATT, Integer.toString (++this.counter));
            partListE.addContent (partE);
            //track source for subunit - if from interactant A
            tempList = XMLUtil.getElement (BINDTags.COMPLEX_MOL_OBJ_SRC_A,
                    bindMolObjList.get (i), XMLUtil.getDepth (bindMolObjList.get (i).getName()));
            if (XMLUtil.validateSingle(BINDTags.COMPLEX_MOL_OBJ_SRC_A, tempList)) {
                interactionRefE = new Element (PSIMITags.INT_REF, this.psins);
                BINDIntID = tempList.get (0).getChildText(BINDTags.INTERACTION_ID).trim();
                if (!interactionIDsList.contains (BINDIntID)) {
                    System.err.println ("Error 5: Wrong interaction ID referenced in complex: " +
                            BINDIntID);
                    this.badInteractionData = true;
                }
                newIntID = matchIntID (BINDIntID);

                //check to see if the id returned is a good referent
                //if(!this.badInteractionData){

                    interactionRefE.setText(newIntID);
                    partE.addContent (interactionRefE);
                    interactantAtt = PSIMITags.ATT_COMPLEX_INT_REF_A;
                //}
            } else {
                //track source for subunit - if from interactant B
                tempList = XMLUtil.getElement (BINDTags.COMPLEX_MOL_OBJ_SRC_B,
                        bindMolObjList.get (i), XMLUtil.getDepth (bindMolObjList.get (i).getName()));
                if (XMLUtil.validateSingle(BINDTags.COMPLEX_MOL_OBJ_SRC_B, tempList)) {
                    interactionRefE = new Element (PSIMITags.INT_REF, this.psins);
                    BINDIntID = tempList.get (0).getChildText(BINDTags.INTERACTION_ID).trim();
                    if (!interactionIDsList.contains (BINDIntID)) {
                        System.err.println ("Error 5: Wrong interaction ID referenced in complex: " +
                                BINDIntID);
                        this.badInteractionData = true;
                    }
                    newIntID = matchIntID (BINDIntID);
                    interactionRefE.setText(newIntID);
                    partE.addContent (interactionRefE);
                    interactantAtt = PSIMITags.ATT_COMPLEX_INT_REF_B;
                } else {
                    //more if action-result is supported for interactions.
                }
            }
            //add interactor details if no reference to interaction available
            Element interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
            translateInteractor (interactorE, bindMolObjList.get (i), ++this.counter, null,null,null);
            if (interactantAtt == null) {
                partE.addContent (interactorE);
            } else {
                addAttribute (partE, PSIMITags.ATT_COMPLEX_INT_SRC, interactantAtt);
                addAttribute (partE, PSIMITags.ATT_COMPLEX_INT_REF_BIND, BINDIntID);
            }
            addAttribute (partE, PSIMITags.ATT_COMPLEX_SUBUNIT_NUM, subUnitNum);
        }
    }

    private String matchIntID (String BINDIntID) {
        //if the interaction is in the current set of interactions.
        //get the correponding psi mi id and return it.
        if (interactionsMap.containsKey(BINDIntID)) {
            return interactionsMap.get (BINDIntID);
        }
        //if the interaction is classified as a bad interaction then
        //return the BIND ID and print error that is it is a bad interaction
        else if (badInteractionsMap.containsKey(BINDIntID)) {
                System.err.println ("Error 6: Bad reference from complex. " +
                   "Interaction does not exist with BINDID: " + BINDIntID);
            this.badInteractionData = true;
                return BINDIntID;
            }
        //if the interaction is not in the current set or in the bad set
        //it isn't in the file.  pint out error and return bind id
        else {
            System.err.println ("Error 7: Bad reference from complex. " +
                   "Interaction does not exist with BINDID: " + BINDIntID);
            this.badInteractionData = true;
            return BINDIntID;
        }
    }


    private void translateInteractors (Element partListE,Element expListE, Element inferredListE,
                               Element BINDInteractionE, int interactionNum, String ComplexID) throws JDOMException, IOException {

        //parse the BIND interaction ID
        ArrayList <Element> eList = XMLUtil.getElement (BINDTags.INTERACTION_ID, BINDInteractionE,
                                    XMLUtil.getDepth (BINDTags.INTERACTION));
        String BINDInteractionID = null;
        if (XMLUtil.validateSingle (BINDTags.INTERACTION_ID, eList) &&
            XMLUtil.validateInt(eList.get (0))) {
            BINDInteractionID = eList.get (0).getTextTrim ();
        }



        //get the sources associated only with the interactions.
        ArrayList<String> interaction_pmids = new ArrayList<String>();
        Element dummyE = new Element ("dummyElement", this.psins);

        //should get pubmed from interaction source first
        translatePubMed(dummyE,BINDInteractionE.getChild (BINDTags.INTERACTION_SOURCE),interaction_pmids );

        //get the places from the interaction --> add the places of the interaction to each interactor
        //check to see if there are places associated with interaction
        ArrayList<Element> additional_compartments = new ArrayList<Element>();
        ArrayList <Element> descrList = XMLUtil.getElement (BINDTags.DESC,
                         BINDInteractionE, XMLUtil.getDepth (BINDTags.INTERACTION));
         if(XMLUtil.validateSingle(BINDTags.DESC,  descrList)){
            Element descE = descrList.get(0);

            //get the places
            ArrayList <Element> placesList = XMLUtil.getElement (BINDTags.PLACE,
                             descE, XMLUtil.getDepth (/*"BIND-descr_place"*/descE.getName()),true);

            if (XMLUtil.validateExists(BINDTags.PLACE, placesList)) {

                //go through each place and see what the BIND-gen-place is
                for(int k = 0; k< placesList.size();k++){

                    //create a new comparment
                    Element compartmentE = new Element(PSIMITags.COMPARTMENT, this.psins);

                    //translate place
                    compartmentE = translatePlace(compartmentE, placesList.get(k),"BIND-descr_place","BINDInteractionID: " +BINDInteractionID,interaction_pmids);

                    //add the compartment to the organism
                    if(compartmentE != null){
                        //add an attribute to the interactor indicating that the compartment was taken from the interaction and not the interactor.
                        addAttribute(compartmentE,PSIMITags.ATT_Int_place,"interaction place");

                        additional_compartments.add(compartmentE);
                    }
                } //for loop - for each place
            }
          }

        //for this interaction create an inferred interaction
        Element inferredE = new Element (PSIMITags.INFERRED, this.psins);


        //1st BIND interactor - create a particpant object and an interactor object
        Element interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
        //partE.addContent (interactorE);

        Element interactor_A = BINDInteractionE.getChild (BINDTags.INTERACT_A);

        int species_A = translateInteractor (interactorE, interactor_A, ++this.counter,interaction_pmids,additional_compartments,null);

        //add the experimental role from Bind-condition_bait-condition
        //can have one of three values: a-is-bait, b-is-bait,not-applicable
        eList = XMLUtil.getElement(BINDTags.BAIT_COND, BINDInteractionE, XMLUtil.getDepth(BINDTags.INTERACTION));
        String bait_condition = "";
        if (XMLUtil.validateSingle (BINDTags.BAIT_COND, eList)) {
            bait_condition = eList.get(0).getAttributeValue("value");
        }


        String participantID = newInteractor(partListE, interactorE);

        //before we add the interactor, check to see that is not already in the list.
        if( participantID == null){
            Element partE = new Element (PSIMITags.PART, this.psins);

            participantID = Integer.toString (++this.counter);

            partE.setAttribute(PSIMITags.ID_ATT, participantID);

            partListE.addContent (partE);

            partE.addContent (interactorE);

            if(bait_condition.equalsIgnoreCase("a-is-bait"))
                addExperimentalRole(partE,"bait");
            else if(bait_condition.equalsIgnoreCase("b-is-bait"))
                addExperimentalRole(partE,"prey");
            else if(bait_condition.equalsIgnoreCase("not-applicable"))
                addExperimentalRole(partE,"none");

            //check to see if there is an experimental form associated with this molecule
           eList = XMLUtil.getElement(BINDTags.EXPFORM_A, BINDInteractionE, XMLUtil.getDepth(BINDTags.INTERACTION));
            if (XMLUtil.validateSingle (BINDTags.EXPFORM_A, eList)) {
                //create experimental Interactor List to store experimental form
                Element experIntListE = new Element (PSIMITags.EXPERINTLIST, this.psins);

                Element experIntE = new Element(PSIMITags.EXPERINT, this.psins);
                experIntListE.addContent(experIntE);


                Element Form_interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
                experIntE.addContent(Form_interactorE);

                //check to see if the experimental form is a BIND-object.  (an experimental form can also be
                //  a profile or a gene.  Only parse the experimental forms that are objects.
                ArrayList <Element> check_type = XMLUtil.getElement(BINDTags.EXPFORM_OBJECT, eList.get(0), XMLUtil.getDepth(BINDTags.EXPFORM_A));
                if (XMLUtil.validateSingle (BINDTags.EXPFORM_OBJECT, check_type)){
                    //translate the experimental form
                    translateInteractor (Form_interactorE, eList.get(0), ++this.counter,interaction_pmids, null, interactor_A);
                    partE.addContent(experIntListE);
                }

            }
        }

        //add the particapnt to the inferred interaction
        Element partIIE = new Element (PSIMITags.PART, this.psins);
        Element partIIRefE = new Element(PSIMITags.PART_REF, this.psins);
        partIIRefE.setText(participantID);
        partIIE.addContent(partIIRefE);
        inferredE.addContent(partIIE);



        //2nd BIND interactor - create a particpant object and an interactor object

        interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
        //partE.addContent (interactorE);

        Element interactor_B = BINDInteractionE.getChild(BINDTags.INTERACT_B);

        int species_B = translateInteractor (interactorE, interactor_B, ++this.counter,interaction_pmids, additional_compartments,null);


        participantID = newInteractor(partListE, interactorE);
         //check to see if there is an experimental form associated with this molecule
        if(participantID == null){
            Element partE = new Element (PSIMITags.PART, this.psins);

            participantID = Integer.toString (++this.counter);
            partE.setAttribute(PSIMITags.ID_ATT, participantID);

            partListE.addContent (partE);

            partE.addContent (interactorE);

            if(bait_condition.equalsIgnoreCase("a-is-bait"))
                addExperimentalRole(partE,"prey");
            else if(bait_condition.equalsIgnoreCase("b-is-bait"))
                addExperimentalRole(partE,"bait");
            else if(bait_condition.equalsIgnoreCase("not-applicable"))
                 addExperimentalRole(partE,"none");


            eList = XMLUtil.getElement(BINDTags.EXPFORM_B, BINDInteractionE, XMLUtil.getDepth(BINDTags.INTERACTION));
            if (XMLUtil.validateSingle (BINDTags.EXPFORM_B, eList)) {
                //create experimental Interactor List to store experimental form
                Element experIntListE = new Element (PSIMITags.EXPERINTLIST, this.psins);

                Element experIntE = new Element(PSIMITags.EXPERINT, this.psins);
                experIntListE.addContent(experIntE);

                Element Form_interactorE = new Element (PSIMITags.INTERACTOR, this.psins);
                experIntE.addContent(Form_interactorE);

                //translate the experimental form
                //check to see if the experimental form is a BIND-object.  (an experimental form can also be
                //  a profile or a gene.  Only parse the experimental forms that are objects.
                ArrayList <Element> check_type = XMLUtil.getElement(BINDTags.EXPFORM_OBJECT, eList.get(0), XMLUtil.getDepth(BINDTags.EXPFORM_B));
                if (XMLUtil.validateSingle (BINDTags.EXPFORM_OBJECT, check_type)){
                    //translate the experimental form
                    translateInteractor (Form_interactorE, eList.get(0), ++this.counter,interaction_pmids, null, interactor_B);
                    partE.addContent(experIntListE);
                }

            }
        }

        //add the particapnt to the inferred interaction
        partIIE = new Element (PSIMITags.PART, this.psins);
        partIIRefE = new Element(PSIMITags.PART_REF, this.psins);
        partIIRefE.setText(participantID);
        partIIE.addContent(partIIRefE);
        inferredE.addContent(partIIE);

        //add inferred interaction to th inferred interaction list.
        inferredListE.addContent(inferredE);


        //check to see if species_A and species_B match
        //in order to transfer this info to the interaction_descr BIND-place they need to agree.
        //TODO : the host species described in the experiment description is not the same as the host system...set to zero.
        int speciesID;
        if(species_A != species_B)
            if(species_A == Integer.parseInt(PSIMITags.TAXID_UN_ID) )
                 speciesID = species_B;
            else if(species_B == Integer.parseInt(PSIMITags.TAXID_UN_ID))
                speciesID = species_A;
            //if they don't equal and neither one of them is the undefined species id use zero as the default species id.
            else
                speciesID = 0;
        else
            speciesID = species_A;

        speciesID = 0;

        //TODO translating ton ly the first experimenatl description will work fine for spoke represented complexes
        //but not for hierarchical clusters.. TODO: FIX
        if(interactionNum==0){
            //parse the experiment and translate experiment description

            //make sure the counter is set to the current number
            //because we are about to go to the parent class.
            current_translator_instance.setCounter(this.counter);

            current_translator_instance.translateExpDesc (expListE, BINDInteractionE, BINDInteractionID, speciesID, interaction_pmids);

            //update the current counter
            this.counter = current_translator_instance.getCounter();

            //for the first interaction find out if the bait is specified.  If not print out the stats
            if(bait_condition.equalsIgnoreCase("not-applicable"))
                error_log.write(ComplexID + "\tComplex is not a pull down\n");
            else
                error_log.write(ComplexID + "\tComplex has a bait defined and is a pull down\n");
        }

    }

    private String newInteractor(Element partListE, Element new_interactor){

        //compare the new Interactor to the list currently in the participant list
        ArrayList <Element> participants = new ArrayList <Element> ();
        participants.addAll(partListE.getChildren());

        //get the new_interactor ifo
        Element new_names = new_interactor.getChild(PSIMITags.NAMES,partListE.getNamespace());
        String new_shortname = "";
        if(new_names != null)
            new_shortname = new_names.getChild(PSIMITags.SHORT_LABEL,partListE.getNamespace()).getValue();
        Element new_XRef = new_interactor.getChild(PSIMITags.XREF,partListE.getNamespace());
        String new_primaryRef = "";
        if(new_XRef != null)
                new_primaryRef = new_XRef.getChild(PSIMITags.PREF,partListE.getNamespace()).getAttributeValue("id");

        //String new_shortname = new_interactor.getChild(PSIMITags.NAMES).getChild(PSIMITags.SHORT_LABEL).getValue();
        //String new_primaryRef = new_interactor.getChild(PSIMITags.XREF).getChild(PSIMITags.PREF).getAttributeValue("id");

        for(int k =0;k<participants.size(); k++){

            //for each participant get it's interactor and compare it to the current
            Element interactor = participants.get(k).getChild(PSIMITags.INTERACTOR,partListE.getNamespace());
            if(interactor != null){
                Element names = interactor.getChild(PSIMITags.NAMES,partListE.getNamespace());

                String shortname = "";
                if(new_names != null)
                    shortname = names.getChild(PSIMITags.SHORT_LABEL,partListE.getNamespace()).getValue();

                Element XRef = interactor.getChild(PSIMITags.XREF,partListE.getNamespace());
                String primaryRef = "";
                if(XRef != null)
                    primaryRef = XRef.getChild(PSIMITags.PREF,partListE.getNamespace()).getAttributeValue("id");

                if(shortname.equalsIgnoreCase(new_shortname) && primaryRef.equalsIgnoreCase(new_primaryRef))
                    return participants.get(k).getAttributeValue("id");
            }
        }


        return null;
    }
}
