package org.pathwaycommons.trans.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.HashSet;

import org.jdom.Element;

import org.genemania.dw.util.GenUtil;
import org.genemania.dw.util.DefParams;
import org.pathwaycommons.trans.BINDTags;

/**
 * Generates basic stats (supports elements and attributes only) and listing of
 * elements that are actually used in an xml file. The stats generator
 * intentionally iterates more than it should to double check sanity of xml.
 *
 * @author rashadbadrawi
 */

public class XMLStats {

    private static final String DEF_STATS_FILE_NAME = "XMLStats.txt";
    private static PrintWriter log = GenUtil.getDefaultLog();

    //create a hash to store all the interaction unique ids
    private HashMap<String, String> interaction_ids = new HashMap<String, String>();

    //create a hash to store all the complex unique ids
    private HashMap<String, String> complex_ids = new HashMap<String, String>();

    private HashSet<String> duplicates = new HashSet<String>();

    private String inputPathName;

    public XMLStats () {}

    public void setInputPathName (String inputPathName) {

        GenUtil.validateString(inputPathName);
        this.inputPathName = inputPathName;
    }

    public String getInputPathName () { return inputPathName; }

    public void generateStats () throws Exception {

        File sourceFileOrDir = new File (getInputPathName());
        if (sourceFileOrDir.isFile ()) {
            String fileName = sourceFileOrDir.getAbsolutePath();
            System.out.print("Stats for: " + fileName);
            Element rootElement = XMLUtil.readSchemaBasedXML(fileName);
            generateStats (rootElement, fileName);
            rootElement = null;                         //attempt cleaning up
            System.gc();
        } else {            //} else if (sourceFileOrDir.isDirectory () {
            File filesArr [] = sourceFileOrDir.listFiles();
            int cnt = 1;
            for (int i = 0; i < filesArr.length; i++) {
                //if (!filesArr [i].getAbsolutePath().endsWith(XMLUtil.XML_FILE_SUFFIX)) {
                if (!filesArr [i].getAbsolutePath().contains (XMLUtil.XML_FILE_SUFFIX)) {
                    continue;
                }
                System.out.println("Stats for: " + cnt++ + " " + filesArr [i]);
                Element rootElement = XMLUtil.readSchemaBasedXML(
                                      filesArr [i].getAbsolutePath());
                generateStats (rootElement, filesArr [i].getAbsolutePath());
                rootElement = null;                     //attempt cleaning up
                System.gc();
           }
        }
    }

    public void generateStats(Element rootElement, String fileName) {
         String id;
        //check to see if this element is an interaction or a complex.
        if(rootElement.getName().equalsIgnoreCase("BIND-Interaction")){
            id = rootElement.getChild("BIND-Interaction_iid").getChildText("Interaction-id");
            //check to see if id has already been entered in hash of interaction ids
            //if we have seen this id before then return
            if(interaction_ids.containsKey(id)){
                duplicates.add(id);
                return;
            }else
                interaction_ids.put(id, id);

        }else if(rootElement.getName().equalsIgnoreCase("BIND-Molecular-Complex")){
             id = rootElement.getChild("BIND-Molecular-Complex_mcid").getChildText("Molecular-Complex-id");
             //check to see if id has already been entered in hash of complex ids
            //if we have seen this id before then return
            if(complex_ids.containsKey(id)){
                duplicates.add(id);
                return;
            }else
                complex_ids.put(id, id);

        }

        new XMLElementInfo (rootElement, fileName);
        ArrayList<Element> childrenList = new ArrayList<Element> (rootElement.getChildren());
        for (int i = 0; i < childrenList.size(); i++) {
            generateStats(childrenList.get(i), fileName);
        }
    }

    public void dumpStats() throws IOException {

        File outputFile = new File (DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP),
                                    XMLStats.DEF_STATS_FILE_NAME);
        BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile));

        File outputFile_uniqueids = new File (DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP),
                                    "interaction_ids.txt");
        BufferedWriter bw2 = new BufferedWriter(new FileWriter(outputFile_uniqueids));

        File outputFile_uniquecomids = new File (DefParams.getDefaultProp(DefParams.TOOLS_PATH_PROP),
                                            "complex_ids.txt");
        BufferedWriter bw3 = new BufferedWriter(new FileWriter(outputFile_uniquecomids));


        Iterator iterator = XMLElementInfo.getAll ().keySet().iterator();
        System.out.println("Total num of unique elements: " + XMLElementInfo.getAll ().size());
        System.out.println("Dumping all unique elements...");
        bw.write (XMLElementInfo.getHeaders());
        bw.newLine ();
        while (iterator.hasNext()) {
            XMLElementInfo xei = XMLElementInfo.getAll ().get ((String)iterator.next());
            bw.write(xei.toString());
            bw.newLine();
        }
        bw.flush ();
        bw.close ();

        bw2.write(interaction_ids.keySet().toString());
        bw2.newLine();
        bw2.write(duplicates.toString());
        bw2.flush();
        bw2.close();

        bw3.write(complex_ids.keySet().toString());
        bw3.flush();
        bw3.close();

        System.out.println("Done...");
    }

    public static void main(String args[]) {

        log = GenUtil.getDefaultLog();
        log.println(GenUtil.getTimeStamp());
        GenUtil.registerStart();
        String usageMsg = "Usage: XMLStats inputpath";
        String warnMsg = "WARNING: Missing command line args, using defaults";
        if (args == null || args.length < 1) {
            log.println(warnMsg);
            log.println(usageMsg);
            System.out.println(warnMsg);
            System.out.println(usageMsg);
            args = DefParams.getCommandLineArgs(XMLStats.class.getName());
        }
        try {
            XMLStats xmlStats = new XMLStats ();
            xmlStats.setInputPathName(args [0]);
            xmlStats.generateStats();
            xmlStats.dumpStats();
        } catch (Throwable e) {
            e.printStackTrace();
            e.printStackTrace(log);
        } finally {
            log.println(GenUtil.getExecTimeStr());
            log.flush();
            log.close();
        }
    }
}
